@@ -73,7 +73,6 @@ from the copyright holder.
 #  include <sys/stropts.h>
 # endif
 # ifdef __EMX__
-#  define link rename
 #  define chown(a,b,c)
 #  include <io.h>
 # endif
@@ -81,17 +80,71 @@ from the copyright holder.
 
 #include <X11/Xlib.h>
 
+Xauth *
+getAuthHelper( unsigned short namelen, const char *name )
+{
+	Xauth *new;
+
+	if (!(new = Malloc( sizeof(*new) )))
+		return 0;
+
+	if (!(new->name = Malloc( namelen ))) {
+		free( new );
+		return 0;
+	}
+	new->name_length = namelen;
+	memmove( new->name, name, namelen );
+
+	new->family = FamilyWild;
+	new->address_length = 0;
+	new->address = 0;
+	new->number_length = 0;
+	new->number = 0;
+
+	return new;
+}
+
+Xauth *
+generateAuthHelper( unsigned short namelen, const char *name, int nbytes )
+{
+	Xauth *new;
+
+	if (!(new = getAuthHelper( namelen, name )))
+		return 0;
+
+	if (!(new->data = Malloc( nbytes ))) {
+		free( new->name );
+		free( new );
+		return 0;
+	}
+	new->data_length = nbytes;
+	if (!generateAuthData( new->data, nbytes )) {
+		free( new->data );
+		free( new->name );
+		free( new );
+		return 0;
+	}
+
+	return new;
+}
+
+#define MIT_AUTH_DATA_LEN 16 /* bytes of authorization data */
+
+static Xauth *
+mitGetAuth( unsigned short namelen, const char *name )
+{
+	return generateAuthHelper( namelen, name, MIT_AUTH_DATA_LEN );
+}
+
 struct AuthProtocol {
 	unsigned short name_length;
 	const char *name;
-	void (*InitAuth)( unsigned short len, const char *name );
 	Xauth *(*GetAuth)( unsigned short len, const char *name );
 #ifdef XDMCP
 	void (*GetXdmcpAuth)( struct protoDisplay *pdpy,
 	                      unsigned short authorizationNameLen,
 	                      const char *authorizationName );
 #endif
-	int inited;
 };
 
 #ifdef XDMCP
@@ -100,28 +153,28 @@ struct AuthProtocol {
 # define xdmcpauth(arg)
 #endif
 
-static struct AuthProtocol authProtocols[] = {
+static const struct AuthProtocol authProtocols[] = {
 { (unsigned short)18, "MIT-MAGIC-COOKIE-1",
-	mitInitAuth, mitGetAuth xdmcpauth(NULL), False
+	mitGetAuth xdmcpauth(0)
 },
 #ifdef HASXDMAUTH
 { (unsigned short)19, "XDM-AUTHORIZATION-1",
-	xdmInitAuth, xdmGetAuth xdmcpauth(xdmGetXdmcpAuth), False
+	xdmGetAuth xdmcpauth(xdmGetXdmcpAuth)
 },
 #endif
 #ifdef SECURE_RPC
 { (unsigned short)9, "SUN-DES-1",
-	secureRPCInitAuth, secureRPCGetAuth xdmcpauth(NULL), False
+	secureRPCGetAuth xdmcpauth(0)
 },
 #endif
 #ifdef K5AUTH
 { (unsigned short)14, "MIT-KERBEROS-5",
-	krb5InitAuth, krb5GetAuth xdmcpauth(NULL), False
+	krb5GetAuth xdmcpauth(0)
 },
 #endif
 };
 
-static struct AuthProtocol *
+static const struct AuthProtocol *
 findProtocol( unsigned short name_length, const char *name )
 {
 	unsigned i;
@@ -132,7 +185,7 @@ findProtocol( unsigned short name_length, const char *name )
 		{
 			return &authProtocols[i];
 		}
-	return (struct AuthProtocol *)0;
+	return 0;
 }
 
 int
@@ -146,15 +199,11 @@ validAuthorization( unsigned short name_length, const char *name )
 static Xauth *
 generateAuthorization( unsigned short name_length, const char *name )
 {
-	struct AuthProtocol *a;
+	const struct AuthProtocol *a;
 	Xauth *auth = 0;
 
 	debug( "generateAuthorization %s\n", name );
 	if ((a = findProtocol( name_length, name ))) {
-		if (!a->inited) {
-			(*a->InitAuth)( name_length, name );
-			a->inited = True;
-		}
 		auth = (*a->GetAuth)( name_length, name );
 		if (auth) {
 			debug( "got %p (%d %.*s) %02[*hhx\n", auth,
@@ -174,16 +223,11 @@ setProtoDisplayAuthorization( struct protoDisplay *pdpy,
                               unsigned short authorizationNameLen,
                               const char *authorizationName )
 {
-	struct AuthProtocol *a;
+	const struct AuthProtocol *a;
 	Xauth *auth;
 
-	a = findProtocol( authorizationNameLen, authorizationName );
 	pdpy->xdmcpAuthorization = pdpy->fileAuthorization = 0;
-	if (a) {
-		if (!a->inited) {
-			(*a->InitAuth)( authorizationNameLen, authorizationName );
-			a->inited = True;
-		}
+	if ((a = findProtocol( authorizationNameLen, authorizationName ))) {
 		if (a->GetXdmcpAuth) {
 			(*a->GetXdmcpAuth)( pdpy, authorizationNameLen, authorizationName );
 			auth = pdpy->xdmcpAuthorization;
@@ -202,6 +246,43 @@ setProtoDisplayAuthorization( struct protoDisplay *pdpy,
 
 #endif /* XDMCP */
 
+SetAuthStatus
+setDynamicDisplayAuthorization( struct display *d,
+                                const char *authorizationName,
+                                const char *authorizationData )
+{
+	Xauth *new;
+	char *data;
+	int dataLen;
+
+	dataLen = strlen( authorizationData ) / 2;
+	if (!(data = Malloc( dataLen )))
+		return SetAuthOOM;
+	if (!hexToBinary( data, authorizationData )) {
+		free( data );
+		return SetAuthBad;
+	}
+
+	if (!(new = getAuthHelper( strlen( authorizationName ),
+	                           authorizationName ))) {
+		free( data );
+		return SetAuthOOM;
+	}
+
+	new->data = data;
+	new->data_length = dataLen;
+
+	if (!(d->authorizations = Malloc( sizeof(Xauth *) ))) {
+		free( new->name );
+		free( new );
+		free( data );
+		return SetAuthOOM;
+	}
+	d->authorizations[0] = new;
+	d->authNum = 1;
+	return SetAuthOk;
+}
+
 void
 cleanUpFileName( const char *src, char *dst, int len )
 {
@@ -292,7 +373,7 @@ saveServerAuthorizations( struct display *d, Xauth **auths, int count )
 		if (!(auth_file = fdOpenW( creat( d->authFile, 0600 ) ))) {
 			logError( "Cannot open X server authorization file %s\n", d->authFile );
 			free( d->authFile );
-			d->authFile = NULL;
+			d->authFile = 0;
 			return False;
 		}
 	} else {
@@ -317,18 +398,20 @@ saveServerAuthorizations( struct display *d, Xauth **auths, int count )
 		 * to the auth file so xrdb and setup programs don't fail.
 		 */
 		if (auths[i]->data_length > 0)
-			if (!XauWriteAuth( auth_file, auths[i] ) ||
-			    fflush( auth_file ) == EOF)
-			{
+			if (!XauWriteAuth( auth_file, auths[i] )) {
 				fclose( auth_file );
-				logError( "Cannot write X server authorization file %s\n",
-				          d->authFile );
-				free( d->authFile );
-				d->authFile = NULL;
-				return False;
+				goto whoops;
 			}
 	}
-	fclose( auth_file );
+	if (fclose( auth_file ) == EOF) {
+	  whoops:
+		logError( "Cannot write X server authorization file %s: %m\n",
+		          d->authFile );
+		unlink( d->authFile );
+		free( d->authFile );
+		d->authFile = 0;
+		return False;
+	}
 	return True;
 }
 
@@ -336,45 +419,33 @@ void
 setLocalAuthorization( struct display *d )
 {
 	Xauth *auth, **auths;
-	int i, j;
+	int i, confAuths, gotAuths;
 
 	if (d->authorizations)
 	{
 		for (i = 0; i < d->authNum; i++)
 			XauDisposeAuth( d->authorizations[i] );
-		free( (char *)d->authorizations );
-		d->authorizations = (Xauth **)NULL;
+		free( d->authorizations );
+		d->authorizations = 0;
 		d->authNum = 0;
 	}
 	debug( "setLocalAuthorization %s, auths %[s\n", d->name, d->authNames );
-	if (!d->authNames)
+	confAuths = arrLen( d->authNames );
+	if (!(auths = Malloc( confAuths * sizeof(Xauth *) )))
 		return;
-	for (i = 0; d->authNames[i]; i++)
-		;
-	d->authNameNum = i;
-	if (d->authNameLens)
-		free( (char *)d->authNameLens );
-	d->authNameLens = (unsigned short *)Malloc( d->authNameNum * sizeof(unsigned short) );
-	if (!d->authNameLens)
-		return;
-	for (i = 0; i < d->authNameNum; i++)
-		d->authNameLens[i] = strlen( d->authNames[i] );
-	auths = (Xauth **)Malloc( d->authNameNum * sizeof(Xauth *) );
-	if (!auths)
-		return;
-	j = 0;
-	for (i = 0; i < d->authNameNum; i++) {
-		auth = generateAuthorization( d->authNameLens[i], d->authNames[i] );
+	gotAuths = 0;
+	for (i = 0; i < confAuths; i++) {
+		auth = generateAuthorization( strlen( d->authNames[i] ), d->authNames[i] );
 		if (auth)
-			auths[j++] = auth;
+			auths[gotAuths++] = auth;
 	}
-	if (saveServerAuthorizations( d, auths, j )) {
+	if (saveServerAuthorizations( d, auths, gotAuths )) {
 		d->authorizations = auths;
-		d->authNum = j;
+		d->authNum = gotAuths;
 	} else {
-		for (i = 0; i < j; i++)
+		for (i = 0; i < gotAuths; i++)
 			XauDisposeAuth( auths[i] );
-		free( (char *)auths );
+		free( auths );
 	}
 }
 
@@ -568,7 +639,7 @@ ifioctl( int fd, int cmd, char *arg )
 	struct strioctl ioc;
 	int ret;
 
-	bzero( (char *)&ioc, sizeof(ioc) );
+	bzero( &ioc, sizeof(ioc) );
 	ioc.ic_cmd = cmd;
 	ioc.ic_timout = 0;
 	if (cmd == SIOCGIFCONF) {
@@ -919,7 +990,7 @@ defineSelf( int fd, int file, int auth, int *ok )
 	if ((hp = gethostbyname( name.nodename ))) {
 		saddr.sa.sa_family = hp->h_addrtype;
 		inetaddr = (struct sockaddr_in *)(&(saddr.sa));
-		memmove( (char *)&(inetaddr->sin_addr), (char *)hp->h_addr,
+		memmove( &inetaddr->sin_addr, hp->h_addr,
 		         (int)hp->h_length );
 		if ((family = convertAddr( &(saddr.sa), &len, &addr )) >= 0)
 			writeAddr( FamilyInternet, sizeof(inetaddr->sin_addr),
@@ -964,7 +1035,7 @@ writeLocalAuth( FILE *file, Xauth *auth, const char *name, int *ok )
 	setAuthNumber( auth, name );
 # ifdef STREAMSCONN
 	fd = t_open( "/dev/tcp", O_RDWR, 0 );
-	t_bind( fd, NULL, NULL );
+	t_bind( fd, 0, 0 );
 	defineSelf( fd, file, auth, ok );
 	t_unbind( fd );
 	t_close( fd );
@@ -1089,15 +1160,12 @@ undoUserAuth( const char *name, const char *new_name )
 static char *
 moveUserAuth( const char *name, char *new_name, char *envname )
 {
-	if (unlink( name ))
-		debug( "unlink %s failed\n", name );
-	if (link( new_name, name )) {
-		debug( "link failed %s %s\n", new_name, name );
-		logError( "Cannot move user authorization into place\n" );
+	if (rename( new_name, name )) {
+		debug( "rename %s => %s failed: %m\n", new_name, name );
+		logError( "Cannot move user authorization into place: %m\n" );
 		envname = new_name;
 	} else {
 		debug( "new authorization moved into place\n" );
-		unlink( new_name );
 	}
 	XauUnlockAuth( name );
 	return envname;
@@ -1132,7 +1200,7 @@ setUserAuthorization( struct display *d )
 				return;
 			/*
 			 * Note, that we don't lock the auth file here, as it's
-			 * temporary - we can assume, that we are the only ones
+			 * temporary - we can assume that we are the only ones
 			 * knowing about this file anyway.
 			 */
 #ifdef HAVE_MKSTEMP
@@ -1202,7 +1270,8 @@ setUserAuthorization( struct display *d )
 		}
 		if (!endUserAuth( old, new, new_name, ok )) {
 			if (!name) {
-				logError( "Cannot save user authorization\n" );
+				/* XXX this should be user-visible */
+				logError( "Cannot save user authorization: %m\n" );
 				return;
 			}
 			undoUserAuth( name, new_name );
