@@ -35,12 +35,14 @@ from the copyright holder.
  */
 
 #include "dm.h"
+#include "dm_auth.h"
 #include "dm_socket.h"
 #include "dm_error.h"
 
 #include <string.h>
 #include <signal.h>
 #include <pwd.h>
+#include <sys/stat.h>
 
 static void
 acceptSock( CtrlRec *cr )
@@ -72,8 +74,7 @@ nukeSock( struct cmdsock *cs )
 {
 	unregisterInput( cs->sock.fd );
 	closeNclearCloseOnFork( cs->sock.fd );
-	if (cs->sock.buffer)
-		free( cs->sock.buffer );
+	free( cs->sock.buffer );
 	free( cs );
 }
 
@@ -386,6 +387,11 @@ static void
 processCtrl( const char *string, int len, int fd, struct display *d )
 {
 #define Reply(t) writer( fd, t, strlen( t ) )
+#ifdef HAVE_VTS
+# define CMD_ACTIVATE "activate\t"
+#else
+# define CMD_ACTIVATE
+#endif
 
 	struct display *di;
 	const char *word;
@@ -416,11 +422,7 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 				Reply( "bootoptions\t" );
 			if (d) {
 				if ((d->displayType & d_location) == dLocal)
-#ifdef HAVE_VTS
-					Reply( "local\tactivate\t" );
-#else
-					Reply( "local\t" );
-#endif
+					Reply( "local\t" CMD_ACTIVATE );
 				if (d->allowShutdown != SHUT_NONE) {
 					if (d->allowShutdown == SHUT_ROOT && d->userSess)
 						Reply( "shutdown root\t" );
@@ -448,11 +450,7 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 				if (anyReserveDisplays())
 					writer( fd, cbuf, sprintf( cbuf, "reserve %d\t",
 					                           idleReserveDisplays() ) );
-#ifdef HAVE_VTS
-				Reply( "login\tactivate\n" );
-#else
-				Reply( "login\n" );
-#endif
+				Reply( CMD_ACTIVATE "resume\tmanage\tlogin\n" );
 			}
 			goto bust;
 		} else if (!strcmp( ar[0], "list" )) {
@@ -642,8 +640,7 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 					goto bust;
 				}
 				if (!sdr.how && !sdr.start) {
-					if (d->sdRec.osname)
-						free( d->sdRec.osname );
+					free( d->sdRec.osname );
 					d->sdRec = sdr;
 				} else {
 					if (sdRec.how && sdRec.force == SHUT_FORCE &&
@@ -661,15 +658,13 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 						goto bust;
 					}
 					if (!sdr.start) {
-						if (d->sdRec.osname)
-							free( d->sdRec.osname );
+						free( d->sdRec.osname );
 						d->sdRec = sdr;
 					} else {
 						if (!sdr.how)
 							cancelShutdown();
 						else {
-							if (sdRec.osname)
-								free( sdRec.osname );
+							free( sdRec.osname );
 							sdRec = sdr;
 						}
 					}
@@ -700,8 +695,7 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 						}
 					}
 					sdr.uid = -1;
-					if (sdRec.osname)
-						free( sdRec.osname );
+					free( sdRec.osname );
 					sdRec = sdr;
 				}
 			}
@@ -769,6 +763,7 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 				}
 				if (ar[5]) {
 					if (!(args = unQuote( ar[5] ))) {
+					  oom:
 						fLog( d, fd, "nomem", "out of memory" );
 						goto bust;
 					}
@@ -805,6 +800,63 @@ processCtrl( const char *string, int len, int fd, struct display *d )
 				default:
 					break;
 				}
+			} else if (!strcmp( ar[0], "resume" )) {
+				if (ar[1])
+					goto exce;
+				wakeDisplays();
+			} else if (!strcmp( ar[0], "manage" )) {
+				if (!ar[1])
+					goto miss;
+				if (*ar[1] == ':') {
+					fLog( d, fd, "bad", "display needs host (try localhost?)" );
+					goto bust;
+				}
+				if (findDisplayByName( ar[1] )) {
+					fLog( d, fd, "exists", "display is already being managed" );
+					goto bust;
+				}
+				if (!(di = newDisplay( ar[1] )))
+					goto oom;
+				di->displayType = dForeign | dPermanent | dFromCommand;
+				if (ar[2]) {
+					if (*ar[2] && !strDup( &di->class2, ar[2] )) {
+						removeDisplay( di );
+						goto oom;
+					}
+					if (ar[3]) {
+						if (!ar[4])
+							goto miss;
+						if (ar[5]) {
+							removeDisplay( di );
+							goto exce;
+						}
+						switch (setDynamicDisplayAuthorization( di, ar[3], ar[4] )) {
+						case SetAuthOOM:
+							removeDisplay( di );
+							goto oom;
+						case SetAuthBad:
+							removeDisplay( di );
+							fLog( d, fd, "bad", "invalid authorization data" );
+							goto bust;
+						case SetAuthOk:
+							break;
+						}
+					}
+				}
+			} else if (!strcmp( ar[0], "unmanage" )) {
+				if (!ar[1])
+					goto miss;
+				if (ar[2])
+					goto exce;
+				if (!(di = findDisplayByName( ar[1] ))) {
+					fLog( d, fd, "noent", "display %s not found", ar[1] );
+					goto bust;
+				}
+				if ((di->displayType & d_origin) != dFromCommand) {
+					fLog( d, fd, "perm", "not an on-demand display" );
+					goto bust;
+				}
+				stopDisplay( di );
 			} else {
 				fLog( d, fd, "nosys", "unknown command" );
 				goto bust;
@@ -850,8 +902,7 @@ handleChan( struct display *d, struct bsock *cs, fd_set *reads )
 			cs->buffer = nbuf;
 			cs->buflen = bl;
 			processCtrl( bufp, llen - 1, cs->fd, d );
-			if (obuf)
-				free( obuf );
+			free( obuf );
 			return 1;
 		} else if (!len) {
 			cs->buflen = -bl;
