@@ -22,7 +22,6 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <QClipboard>
 #include <QtDBus/QDBusConnection>
 
 #include <kaboutdata.h>
@@ -37,13 +36,13 @@
 #include <kactioncollection.h>
 #include <ktoggleaction.h>
 #include <KConfigSkeleton>
+#include <ktextedit.h>
 
 #include "configdialog.h"
 #include "klipper.h"
 #include "klippersettings.h"
 #include "urlgrabber.h"
 #include "version.h"
-#include "clipboardpoll.h"
 #include "history.h"
 #include "historyitem.h"
 #include "historystringitem.h"
@@ -51,10 +50,13 @@
 
 #include <zlib.h>
 
+#ifdef Q_WS_X11
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
+#endif
 
 //#define NOISY_KLIPPER
+#include <KPassivePopup>
 
 namespace {
     /**
@@ -124,8 +126,10 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     updateTimestamp(); // read initial X user time
     m_clip = kapp->clipboard();
 
+    connect( m_clip, SIGNAL(changed(QClipboard::Mode)),
+             this, SLOT( newClipData( QClipboard::Mode) ) );
+
     connect( &m_overflowClearTimer, SIGNAL( timeout()), SLOT( slotClearOverflow()));
-    m_overflowClearTimer.start( 1000 );
 
     m_pendingCheckTimer.setSingleShot( true );
     connect( &m_pendingCheckTimer, SIGNAL( timeout()), SLOT( slotCheckPending()));
@@ -143,8 +147,7 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     m_clearHistoryAction = m_collection->addAction( "clearHistoryAction" );
     m_clearHistoryAction->setIcon( KIcon("edit-clear-history") );
     m_clearHistoryAction->setText( i18n("C&lear Clipboard History") );
-    connect(m_clearHistoryAction, SIGNAL(triggered() ), history(), SLOT( slotClear() ));
-    connect( m_clearHistoryAction, SIGNAL( triggered() ), SLOT( slotClearClipboard() ) );
+    connect(m_clearHistoryAction, SIGNAL(triggered() ), SLOT( slotAskClearHistory() ));
     //m_clearHistoryAction->setGroup( defaultGroup );
     m_configureAction = m_collection->addAction( "configureAction" );
     m_configureAction->setIcon( KIcon("configure") );
@@ -179,20 +182,34 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     // if it's too old, drop it and just use load history
     readProperties(m_config.data());
 
-    m_poll = new ClipboardPoll;
-    connect( m_poll, SIGNAL( clipboardChanged( bool ) ),
-             this, SLOT( newClipData( bool ) ) );
-
-    QAction *a = m_collection->addAction("show_klipper_popup");
+    KAction* a = m_collection->addAction("show_klipper_popup");
     a->setText(i18n("Show Klipper Popup-Menu"));
-    qobject_cast<KAction*>(a)->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_V));
+    a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_V));
     connect(a, SIGNAL(triggered()), SLOT(slotPopupMenu()));
 
-    a = m_collection->addAction("repeat_action");
-    a->setText(i18n("Manually Invoke Action on Current Clipboard"));
-    qobject_cast<KAction*>(a)->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_R));
-    connect(a, SIGNAL(triggered()), SLOT(slotRepeatAction()));
+    m_repeatAction = m_collection->addAction("repeat_action");
+    m_repeatAction->setText(i18n("Manually Invoke Action on Current Clipboard"));
+    qobject_cast<KAction*>(m_repeatAction)->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_R));
+    connect(m_repeatAction, SIGNAL(triggered()), SLOT(slotRepeatAction()));
 
+    // add an edit-possibility
+    m_editAction = m_collection->addAction("edit_clipboard");
+    m_editAction->setIcon(KIcon("document-properties"));
+    m_editAction->setText(i18n("&Edit Contents..."));
+    qobject_cast<KAction*>(m_editAction)->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_E), KAction::DefaultShortcut);
+    connect(m_editAction, SIGNAL(triggered()), SLOT(slotEditData()));
+
+    // Cycle through history
+    m_cycleNextAction = m_collection->addAction("cycleNextAction");
+    m_cycleNextAction->setText(i18n("Next history item"));
+    m_cycleNextAction->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_Down), KAction::DefaultShortcut);
+    connect(m_cycleNextAction, SIGNAL(triggered(bool)), SLOT(slotCycleNext()));
+    m_cyclePrevAction = m_collection->addAction("cyclePrevAction");
+    m_cyclePrevAction->setText(i18n("Previous history item"));
+    m_cyclePrevAction->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_Up), KAction::DefaultShortcut);
+    connect(m_cyclePrevAction, SIGNAL(triggered(bool)), SLOT(slotCyclePrev()));
+
+    // Actions toggle
     m_toggleURLGrabAction->setText(i18n("Enable Clipboard Actions"));
     m_toggleURLGrabAction->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_X));
 
@@ -207,6 +224,8 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     popup->plugAction( m_toggleURLGrabAction );
     popup->plugAction( m_clearHistoryAction );
     popup->plugAction( m_configureAction );
+    popup->plugAction( m_repeatAction );
+    popup->plugAction( m_editAction );
     if ( !isApplet() ) {
         popup->plugAction( m_quitAction );
     }
@@ -214,7 +233,6 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
 
 Klipper::~Klipper()
 {
-    delete m_poll;
     delete m_session_managed;
     delete m_showTimer;
     delete m_hideTimer;
@@ -285,23 +303,34 @@ void Klipper::loadSettings()
     m_bKeepContents = KlipperSettings::keepClipboardContents();
     m_bReplayActionInHistory = KlipperSettings::replayActionInHistory();
     m_bNoNullClipboard = KlipperSettings::preventEmptyClipboard();
+    // 0 is the id of "Ignore selection" radiobutton
     m_bIgnoreSelection = KlipperSettings::ignoreSelection();
     m_bIgnoreImages = KlipperSettings::ignoreImages();
-    // 0 is the id of "Synchronize contents" radiobutton
-    m_bSynchronize = ( KlipperSettings::synchronize() == 0 );
+    m_bSynchronize = KlipperSettings::syncClipboards();
     // NOTE: not used atm - kregexpeditor is not ported to kde4
     m_bUseGUIRegExpEditor = KlipperSettings::useGUIRegExpEditor();
-
-    // NOTE (during kconfigxt porting): this option is somehow lost,
-    // no setting for it exists in gui. Or is it a hidden feature?
-    // find it and resolve it. keeping it for now.
     m_bSelectionTextOnly = KlipperSettings::selectionTextOnly();
 
     m_bURLGrabber = KlipperSettings::uRLGrabberEnabled();
     // this will cause it to loadSettings too
     setURLGrabberEnabled(m_bURLGrabber);
-
     history()->setMaxSize( KlipperSettings::maxClipItems() );
+    // Convert 4.3 settings
+    if (KlipperSettings::synchronize() != 3) {
+      // 2 was the id of "Ignore selection" radiobutton
+      m_bIgnoreSelection = KlipperSettings::synchronize() == 2;
+      // 0 was the id of "Synchronize contents" radiobutton
+      m_bSynchronize = KlipperSettings::synchronize() == 0;
+      KConfigSkeletonItem* item = KlipperSettings::self()->findItem("SyncClipboards");
+      item->setProperty(m_bSynchronize);
+      item = KlipperSettings::self()->findItem("IgnoreSelection");
+      item->setProperty(m_bIgnoreSelection);
+      item =  KlipperSettings::self()->findItem("Synchronize"); // Mark property as converted.
+      item->setProperty(3);
+      KlipperSettings::self()->writeConfig();
+      KlipperSettings::self()->readConfig();
+
+    }
 }
 
 void Klipper::saveSettings() const
@@ -434,10 +463,12 @@ void Klipper::saveHistory(bool empty) {
     history_stream << klipper_version; // const char*
 
     if (!empty) {
-        History::iterator it = history()->youngest();
-        while (it.hasNext()) {
-            const HistoryItem *item = it.next();
-            history_stream << item;
+        const HistoryItem *item = history()->first();
+        if (item) {
+            do {
+                history_stream << item;
+                item = history()->find(item->next_uuid());
+            } while (item != history()->first());
         }
     }
 
@@ -537,6 +568,7 @@ void Klipper::slotQuit()
 void Klipper::slotPopupMenu() {
     KlipperPopup* popup = history()->popup();
     popup->ensureClean();
+    popup->slotSetTopActive();
     showPopupMenu( popup );
 }
 
@@ -630,7 +662,7 @@ HistoryItem* Klipper::applyClipChanges( const QMimeData* clipData )
 
 }
 
-void Klipper::newClipData( bool selectionMode )
+void Klipper::newClipData( QClipboard::Mode mode )
 {
     if ( m_locklevel ) {
         return;
@@ -639,7 +671,7 @@ void Klipper::newClipData( bool selectionMode )
     if( blockFetchingNewData())
         return;
 
-    checkClipData( selectionMode );
+    checkClipData( mode == QClipboard::Selection ? true : false );
 
 }
 
@@ -663,6 +695,7 @@ const int MAX_CLIPBOARD_CHANGES = 10; // max changes per second
 
 bool Klipper::blockFetchingNewData()
 {
+#ifdef Q_WS_X11
 // Hacks for #85198 and #80302.
 // #85198 - block fetching new clipboard contents if Shift is pressed and mouse is not,
 //   this may mean the user is doing selection using the keyboard, in which case
@@ -686,8 +719,11 @@ bool Klipper::blockFetchingNewData()
         return true;
     }
     m_pendingContentsCheck = false;
+    if ( m_overflowCounter == 0 ) 
+        m_overflowClearTimer.start( 1000 );
     if( ++m_overflowCounter > MAX_CLIPBOARD_CHANGES )
         return true;
+#endif
     return false;
 }
 
@@ -697,7 +733,7 @@ void Klipper::slotCheckPending()
         return;
     m_pendingContentsCheck = false; // blockFetchingNewData() will be called again
     updateTimestamp();
-    newClipData( true ); // always selection
+    newClipData( QClipboard::Selection ); // always selection
 }
 
 void Klipper::checkClipData( bool selectionMode )
@@ -755,8 +791,15 @@ void Klipper::checkClipData( bool selectionMode )
     // TODO: Rewrite to Qt4 !!!
     //int lastSerialNo = selectionMode ? m_lastSelection : m_lastClipboard;
     //bool changed = data->serialNumber() != lastSerialNo;
-    bool changed = true; // ### FIXME
+    bool changed = true; // ### FIXME (only relevant under polling, might be better to simply remove polling and rely on XFixes)
     bool clipEmpty = data->formats().isEmpty();
+    if (clipEmpty) {
+        // Might be a timeout. Try again
+        clipEmpty = data->formats().isEmpty();
+#ifdef NOISY_KLIPPER
+        kDebug() << "was empty. Retried, now " << (clipEmpty?" still empty":" no longer empty");
+#endif
+    }
 
     if ( changed && clipEmpty && m_bNoNullClipboard ) {
         const HistoryItem* top = history()->first();
@@ -856,10 +899,12 @@ void Klipper::setClipboard( const HistoryItem& item, int mode )
 
 void Klipper::slotClearOverflow()
 {
+    m_overflowClearTimer.stop();
+
     if( m_overflowCounter > MAX_CLIPBOARD_CHANGES ) {
         kDebug() << "App owning the clipboard/selection is lame";
         // update to the latest data - this unfortunately may trigger the problem again
-        newClipData( true ); // Always the selection.
+        newClipData( QClipboard::Selection ); // Always the selection.
     }
     m_overflowCounter = 0;
 }
@@ -867,11 +912,12 @@ void Klipper::slotClearOverflow()
 QStringList Klipper::getClipboardHistoryMenu()
 {
     QStringList menu;
-
-    History::iterator it = history()->youngest();
-    while (it.hasNext()) {
-        const HistoryItem *item = it.next();
-        menu << item->text();
+    const HistoryItem* item = history()->first();
+    if (item) {
+        do {
+            menu << item->text();
+            item = history()->find(item->next_uuid());
+        } while (item != history()->first());
     }
 
     return menu;
@@ -879,13 +925,14 @@ QStringList Klipper::getClipboardHistoryMenu()
 
 QString Klipper::getClipboardHistoryItem(int i)
 {
-    History::iterator it = history()->youngest();
-    while (it.hasNext()) {
-        const HistoryItem *item = it.next();
-        if ( i == 0 ) {
-            return item->text();
-        }
-        i--;
+    const HistoryItem* item = history()->first();
+    if (item) {
+        do {
+            if (i-- == 0) {
+                return item->text();
+            }
+            item = history()->find(item->next_uuid());
+        } while (item != history()->first());
     }
     return QString();
 
@@ -913,6 +960,7 @@ bool Klipper::ignoreClipboardChanges() const
     return false;
 }
 
+#ifdef Q_WS_X11
 // QClipboard uses qt_x_time as the timestamp for selection operations.
 // It is updated mainly from user actions, but Klipper polls the clipboard
 // without any user action triggering it, so qt_x_time may be old,
@@ -957,9 +1005,11 @@ static Bool update_x_time_predicate( Display*, XEvent* event, XPointer )
     }
     return False;
 }
+#endif
 
 void Klipper::updateTimestamp()
 {
+#ifdef Q_WS_X11
     static QWidget* w = 0;
     if ( !w )
         w = new QWidget;
@@ -977,6 +1027,7 @@ void Klipper::updateTimestamp()
     QX11Info::setAppTime( next_x_time );
     XEvent ev; // remove the PropertyNotify event from the events queue
     XWindowEvent( QX11Info::display(), w->winId(), PropertyChangeMask, &ev );
+#endif
 }
 
 static const char * const description =
@@ -1036,4 +1087,94 @@ static void ensureGlobalSyncOff(KSharedConfigPtr config) {
 
 }
 
+void Klipper::slotEditData()
+{
+    const HistoryStringItem* item = dynamic_cast<const HistoryStringItem*>(m_history->first());
+
+    KDialog dlg;
+    dlg.setModal( true );
+    dlg.setCaption( i18n("Edit Contents") );
+    dlg.setButtons( KDialog::Ok | KDialog::Cancel );
+
+    KTextEdit *edit = new KTextEdit( &dlg );
+    if (item) {
+        edit->setText( item->text() );
+    }
+    edit->setFocus();
+    edit->setMinimumSize( 300, 40 );
+    dlg.setMainWidget( edit );
+    dlg.adjustSize();
+
+    if ( dlg.exec() == KDialog::Accepted ) {
+        QString text = edit->toPlainText();
+        if (item) {
+            m_history->remove( item );
+        }
+        m_history->insert( new HistoryStringItem(text) );
+        if (m_myURLGrabber) {
+            m_myURLGrabber->checkNewData( m_history->first() );
+        }
+    }
+
+}
+
+void Klipper::slotAskClearHistory()
+{
+    int clearHist = KMessageBox::questionYesNo(0, 
+                                               i18n("Really delete entire clipboard history?"),
+                                               i18n("Delete clipboard history?"), 
+                                               KStandardGuiItem::yes(),
+                                               KStandardGuiItem::no(),
+                                               QString::fromUtf8("really_clear_history"),
+                                               KMessageBox::Dangerous);
+    if (clearHist == KMessageBox::Yes) {
+      history()->slotClear();
+      slotClearClipboard();
+      saveHistory();
+    }
+
+}
+
+void Klipper::slotCycleNext()
+{
+    m_history->cycleNext();
+    emit passivePopup(i18n("Clipboard history"), cycleText());
+}
+
+void Klipper::slotCyclePrev()
+{
+    m_history->cyclePrev();
+    emit passivePopup(i18n("Clipboard history"), cycleText());
+}
+
+QString Klipper::cycleText() const
+{
+    const HistoryItem* itemprev = m_history->prevInCycle();
+    const HistoryItem* item = m_history->first();
+    const HistoryItem* itemnext = m_history->nextInCycle();
+    QFontMetrics font_metrics(m_history->popup()->fontMetrics());
+    QString rv("<table>");
+    if (itemprev) {
+        rv += "<tr><td>";
+        rv += i18n("up");
+        rv += "</td><td>";
+        rv += font_metrics.elidedText(Qt::escape(itemprev->text().simplified()), Qt::ElideMiddle, 400);
+        rv += "</td></tr>";
+    }
+    rv += "<tr><td>";
+    rv += i18n("current");
+    rv += "</td><td><b>";
+    rv += font_metrics.elidedText(Qt::escape(item->text().simplified()), Qt::ElideMiddle, 400);
+    rv += "</b></td></tr>";
+    if (itemnext) {
+        rv += "<tr><td>";
+        rv += i18n("down");
+        rv += "</td><td>";
+        rv += font_metrics.elidedText(Qt::escape(itemnext->text().simplified()), Qt::ElideMiddle, 400);
+        rv += "</td></tr>";
+    }
+    rv += "</table>";
+    return rv;
+}
+
 #include "klipper.moc"
