@@ -25,18 +25,19 @@
 #include <QKeyEvent>
 #include <QMenu>
 #include <QTimer>
+#include <QClipboard>
 
 #include <KApplication>
 #include <KIcon>
 
 #include "urlgrabber.h"
 
+class KAction;
 class QClipboard;
 class KToggleAction;
 class KAboutData;
 class KActionCollection;
 class URLGrabber;
-class ClipboardPoll;
 class QTime;
 class History;
 class QAction;
@@ -81,6 +82,9 @@ public Q_SLOTS:
     void saveSession();
     void slotHistoryTopChanged();
     void slotConfigure();
+    void slotEditData();
+    void slotCycleNext();
+    void slotCyclePrev();
 
 protected:
     /**
@@ -131,9 +135,12 @@ protected:
     KSharedConfigPtr config() const { return m_config; }
     bool isApplet() const { return m_config != KGlobal::config(); }
 
+Q_SIGNALS:
+    void passivePopup(const QString& caption, const QString& text);
+
 public Q_SLOTS:
     void slotPopupMenu();
-
+    void slotAskClearHistory();
 protected Q_SLOTS:
     void showPopupMenu( QMenu * );
     void slotRepeatAction();
@@ -141,7 +148,7 @@ protected Q_SLOTS:
     void disableURLGrabber();
 
 private Q_SLOTS:
-    void newClipData( bool selectionMode );
+    void newClipData( QClipboard::Mode );
     void slotClearClipboard();
 
     void slotSelectionChanged() {
@@ -173,8 +180,12 @@ private:
     int m_overflowCounter;
     KToggleAction *m_toggleURLGrabAction;
     QAction* m_clearHistoryAction;
+    QAction* m_repeatAction;
+    QAction* m_editAction;
     QAction* m_configureAction;
     QAction* m_quitAction;
+    KAction* m_cycleNextAction;
+    KAction* m_cyclePrevAction;
     QPixmap m_pixmap;
     bool m_bPopupAtMouse :1;
     bool m_bKeepContents :1;
@@ -203,10 +214,10 @@ private:
     QTimer m_overflowClearTimer;
     QTimer m_pendingCheckTimer;
     bool m_pendingContentsCheck;
-    ClipboardPoll* m_poll;
     static KAboutData* m_about_data;
 
     bool blockFetchingNewData();
+    QString cycleText() const;
     KlipperSessionManager* m_session_managed;
     KActionCollection *m_collection;
 };
