@@ -22,6 +22,7 @@
 
 #include <QAction>
 #include <QApplication>
+#include <QClipboard>
 #include <QDesktopWidget>
 #include <QGraphicsView>
 #include <QHBoxLayout>
@@ -31,6 +32,7 @@
 #include <QToolButton>
 #include <QVBoxLayout>
 
+#include <KAction>
 #include <KActionCollection>
 #include <KHistoryComboBox>
 #include <KCompletion>
@@ -42,16 +44,18 @@
 #include <KGlobalSettings>
 #include <KPushButton>
 #include <KTitleWidget>
-#include <KWindowSystem>
 
 #include <Plasma/AbstractRunner>
 #include <Plasma/RunnerManager>
 #include <Plasma/Theme>
 #include <Plasma/Svg>
 
+#include "krunnerapp.h"
 #include "krunnersettings.h"
 #include "interfaces/default/resultscene.h"
 #include "interfaces/default/resultitem.h"
+#include "interfaces/default/krunnertabfilter.h"
+#include "interfaces/default/resultsview.h"
 
 static const int MIN_WIDTH = 420;
 
@@ -90,17 +94,34 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     bottomLayout->addWidget( m_optionsButton );
     */
 
+    //Set up the system activity button, using the krunner global action, showing the global shortcut in the tooltip
     m_activityButton = new QToolButton(m_buttonContainer);
-//    m_activityButton->setDefault(false);
-//    m_activityButton->setAutoDefault(false);
-    m_activityButton->setText(i18n("Show System Activity"));
-    m_activityButton->setToolTip(i18n("Show System Activity"));
-    m_activityButton->setIcon(m_iconSvg->pixmap("status"));
-    connect(m_activityButton, SIGNAL(clicked()), qApp, SLOT(showTaskManager()));
-    connect(m_activityButton, SIGNAL(clicked()), this, SLOT(close()));
+    KRunnerApp *krunnerApp = KRunnerApp::self();
+    QAction *showSystemActivityAction = krunnerApp->actionCollection()->action("Show System Activity");
+    m_activityButton->setDefaultAction(showSystemActivityAction);
+
+    updateSystemActivityToolTip();
+    connect(showSystemActivityAction, SIGNAL(globalShortcutChanged(const QKeySequence &)), this, SLOT(updateSystemActivityToolTip()));
+    connect(showSystemActivityAction, SIGNAL(triggered(bool)), this, SLOT(close()));
     bottomLayout->addWidget(m_activityButton);
     //bottomLayout->addStretch(10);
 
+    m_singleRunnerIcon = new QLabel();
+    bottomLayout->addWidget(m_singleRunnerIcon);
+    m_singleRunnerDisplayName = new QLabel();
+    bottomLayout->addWidget(m_singleRunnerDisplayName);
+
+    m_helpButton = new QToolButton(m_buttonContainer);
+    m_helpButton->setText(i18n("Help"));
+    m_helpButton->setToolTip(i18n("Information on using this application"));
+    m_helpButton->setIcon(m_iconSvg->pixmap("help"));
+    connect(m_helpButton, SIGNAL(clicked(bool)), SLOT(showHelp()));
+    connect(m_helpButton, SIGNAL(clicked(bool)), SLOT(configCompleted()));
+    bottomLayout->addWidget(m_helpButton);
+
+    QSpacerItem* closeButtonSpacer = new QSpacerItem(0,0,QSizePolicy::MinimumExpanding,QSizePolicy::Fixed);
+    bottomLayout->addSpacerItem(closeButtonSpacer);
+
     m_closeButton = new QToolButton(m_buttonContainer);
     KGuiItem guiItem = KStandardGuiItem::close();
     m_closeButton->setText(guiItem.text());
@@ -114,19 +135,20 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     m_layout->addWidget(m_buttonContainer);
 
     m_searchTerm = new KHistoryComboBox(false, w);
+    m_searchTerm->setPalette(QApplication::palette());
     m_searchTerm->setDuplicatesEnabled(false);
 
     KLineEdit *lineEdit = new KLineEdit(m_searchTerm);
     QAction *focusEdit = new QAction(this);
     focusEdit->setShortcut(Qt::Key_F6);
-    
+
     // in therory, the widget should detect the direction from the content
     // but this is not available in Qt4.4/KDE 4.2, so the best default for this widget
     // is LTR: as it's more or less a "command line interface"
     // FIXME remove this code when KLineEdit has automatic direction detection of the "paragraph"
     m_searchTerm->setLayoutDirection( Qt::LeftToRight );
-    
-    connect(focusEdit, SIGNAL(triggered(bool)), lineEdit, SLOT(setFocus()));
+
+    connect(focusEdit, SIGNAL(triggered(bool)), this, SLOT(searchTermSetFocus()));
     addAction(focusEdit);
 
     // the order of these next few lines if very important.
@@ -140,88 +162,134 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     lineEdit->setClearButtonShown(true);
     QStringList pastQueryItems = KRunnerSettings::pastQueries();
     m_searchTerm->setHistoryItems(pastQueryItems);
+    m_searchTerm->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLengthWithIcon);
     m_completion->insertItems(pastQueryItems);
-    bottomLayout->insertWidget(2, m_searchTerm, 10);
+    bottomLayout->insertWidget(4, m_searchTerm, 10);
 
-    m_statusLayout = new QHBoxLayout();
-    m_descriptionLabel = new QLabel(w);
-    m_descriptionLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
-    m_descriptionLabel->hide();
-    m_statusLayout->addWidget(m_descriptionLabel, 10, Qt::AlignLeft | Qt::AlignTop);
+    m_singleRunnerSearchTerm = new KLineEdit(w);
+    bottomLayout->insertWidget(4, m_singleRunnerSearchTerm, 10 );
 
-    m_previousPage = new QLabel(w);
-    m_previousPage->setText("<a href=\"prev\">&lt;&lt;</a>");
-    m_previousPage->hide();
-    m_statusLayout->addWidget(m_previousPage, 0, Qt::AlignLeft | Qt::AlignTop);
+    m_resultsContainer = new QWidget(w);
+    QVBoxLayout* resultsLayout = new QVBoxLayout(m_resultsContainer);
+    resultsLayout->setMargin(0);
 
-    m_nextPage = new QLabel(w);
-    m_nextPage->setText("<a href=\"next\">&gt;&gt;</a>");
-    m_nextPage->hide();
-    m_statusLayout->addWidget(m_nextPage, 0, Qt::AlignLeft | Qt::AlignTop);
-
-    m_layout->addLayout(m_statusLayout);
-
-    m_dividerLine = new QWidget(w);
+    m_dividerLine = new QWidget(m_resultsContainer);
     m_dividerLine->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Minimum);
     m_dividerLine->setFixedHeight(1);
     m_dividerLine->setAutoFillBackground(true);
-    m_dividerLine->hide();
-    m_layout->addWidget(m_dividerLine);
+    resultsLayout->addWidget(m_dividerLine);
 
-    m_resultsView = new QGraphicsView(w);
-    m_resultsView->setFrameStyle(QFrame::NoFrame);
-    m_resultsView->viewport()->setAutoFillBackground(false);
-    m_resultsView->setInteractive(true);
-    m_resultsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_resultsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_resultsView->setOptimizationFlag(QGraphicsView::DontSavePainterState);
-    m_resultsView->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    m_resultsView = new ResultsView(m_resultsContainer);
 
     //kDebug() << "size:" << m_resultsView->size() << m_resultsView->minimumSize();
-    m_resultsScene = new ResultScene(runnerManager, this);
+    m_resultsScene = new ResultScene(runnerManager, m_searchTerm, this);
     m_resultsView->setScene(m_resultsScene);
     m_resultsView->setMinimumSize(m_resultsScene->minimumSizeHint());
+
     connect(m_resultsScene, SIGNAL(matchCountChanged(int)), this, SLOT(matchCountChanged(int)));
     connect(m_resultsScene, SIGNAL(itemActivated(ResultItem *)), this, SLOT(run(ResultItem *)));
-    connect(m_resultsScene, SIGNAL(itemHoverEnter(ResultItem *)), this, SLOT(updateDescriptionLabel(ResultItem *)));
-    connect(m_resultsScene, SIGNAL(itemHoverLeave(ResultItem *)), m_descriptionLabel, SLOT(clear()));
-    connect(m_previousPage, SIGNAL(linkActivated(const QString&)), m_resultsScene, SLOT(previousPage()));
-    connect(m_nextPage, SIGNAL(linkActivated(const QString&)), m_resultsScene, SLOT(nextPage()));
+    connect(m_resultsScene, SIGNAL(ensureVisibility(QGraphicsItem *)), this, SLOT(ensureVisibility(QGraphicsItem *)));
+    connect(m_resultsScene, SIGNAL(actionTriggered()), this, SLOT(actionTriggered()));
 
-    m_layout->addWidget(m_resultsView);
+    resultsLayout->addWidget(m_resultsView);
+
+    m_layout->addWidget(m_resultsContainer);
 
     connect(lineEdit, SIGNAL(userTextChanged(QString)), this, SLOT(queryTextEdited(QString)));
     connect(m_searchTerm, SIGNAL(returnPressed()), this, SLOT(runDefaultResultItem()));
+    connect(m_singleRunnerSearchTerm, SIGNAL(textChanged(QString)), this, SLOT(queryTextEdited(QString)));
+    connect(m_singleRunnerSearchTerm, SIGNAL(returnPressed()),  this, SLOT(runDefaultResultItem()));
+
+    KrunnerTabFilter *krunnerTabFilter = new KrunnerTabFilter(m_resultsScene, lineEdit, this);
+    m_searchTerm->installEventFilter(krunnerTabFilter);
 
     themeUpdated();
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
 
-    new QShortcut(QKeySequence( Qt::Key_Escape ), this, SLOT(close()));
+    new QShortcut(QKeySequence(Qt::Key_Escape), this, SLOT(close()));
+
+    m_layout->setAlignment(Qt::AlignTop);
 
-    m_layout->addStretch(1);
     setTabOrder(0, m_configButton);
     setTabOrder(m_configButton, m_activityButton);
     setTabOrder(m_activityButton, m_searchTerm);
-    setTabOrder(m_searchTerm, m_previousPage);
-    setTabOrder(m_previousPage, m_nextPage);
-    setTabOrder(m_nextPage, m_resultsView);
-    setTabOrder(m_resultsView, m_closeButton);
+    setTabOrder(m_searchTerm, m_resultsView);
+    setTabOrder(m_resultsView, m_helpButton);
+    setTabOrder(m_helpButton, m_closeButton);
+
+    //kDebug() << "size:" << m_resultsView->size() << m_resultsView->minimumSize() << minimumSizeHint();
 
-    //kDebug() << "size:" << m_resultsView->size() << m_resultsView->minimumSize();
     // we restore the original size, which will set the results view back to its
     // normal size, then we hide the results view and resize the dialog
+
     setMinimumSize(QSize(MIN_WIDTH , 0));
+    adjustSize();
+
+    // we load the last used size; the saved value is the size of the dialog when the
+    // results are visible;
+
     if (KGlobal::config()->hasGroup("Interface")) {
         KConfigGroup interfaceConfig(KGlobal::config(), "Interface");
         restoreDialogSize(interfaceConfig);
     }
 
-    //kDebug() << "size:" << minimumSizeHint() << minimumSize();
+    m_defaultSize = size();
+    m_resultsContainer->hide();
+
+    m_delayedQueryTimer.setSingleShot(true);
+    m_delayedQueryTimer.setInterval(100);
+    connect(&m_delayedQueryTimer, SIGNAL(timeout()), this, SLOT(delayedQueryLaunch()));
+
     QTimer::singleShot(0, this, SLOT(resetInterface()));
 }
+void Interface::updateSystemActivityToolTip()
+{
+    /* Set the tooltip for the Show System Activity button to include the global shortcut */
+    KRunnerApp *krunnerApp = KRunnerApp::self();
+    KAction *showSystemActivityAction = dynamic_cast<KAction *>(krunnerApp->actionCollection()->action("Show System Activity"));
+    if(!showSystemActivityAction)
+        return;
+
+    QString shortcut = showSystemActivityAction->globalShortcut().toString();
+    if(!shortcut.isEmpty())
+        m_activityButton->setToolTip( i18nc("tooltip, shortcut", "%1 (%2)", showSystemActivityAction->toolTip(), shortcut));
+    else
+        m_activityButton->setToolTip( showSystemActivityAction->toolTip() );
+}
+void Interface::setConfigWidget(QWidget *w)
+{
+    m_resultsView->hide();
+    m_searchTerm->setEnabled(false);
+    m_layout->addWidget(w);
+    resize(m_defaultSize);
+    connect(w, SIGNAL(destroyed(QObject*)), this, SLOT(configWidgetDestroyed()));
+}
+
+void Interface::configWidgetDestroyed()
+{
+    QTimer::singleShot(0, this, SLOT(cleanupAfterConfigWidget()));
+}
+
+void Interface::cleanupAfterConfigWidget()
+{
+    m_resultsView->show();
+    resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
+    m_searchTerm->setEnabled(true);
+    m_searchTerm->setFocus();
+}
 
 void Interface::resizeEvent(QResizeEvent *event)
 {
+    // We set m_defaultSize only when the event is spontaneous, i.e. when the user resizes the window
+    // We always update the width, but we update the height only if the resultsContainer is visible.
+
+    if (event->spontaneous() || isManualResizing()) {
+        m_defaultSize.setWidth(width());
+        if (m_resultsContainer->isVisible()) {
+            m_defaultSize.setHeight(height());
+        }
+    }
+
     Plasma::Theme *theme = Plasma::Theme::defaultTheme();
     int gradientWidth = contentsRect().width() - KDialog::marginHint()*2;
     QLinearGradient gr(0, 0, gradientWidth, 0);
@@ -234,7 +302,9 @@ void Interface::resizeEvent(QResizeEvent *event)
         p.setBrush(QPalette::Background, gr);
         m_dividerLine->setPalette(p);
     }
-    m_resultsScene->resize(m_resultsView->width(), m_resultsView->height());
+
+    m_resultsScene->resize(m_resultsView->width(), qMax(m_resultsView->height(), int(m_resultsScene->height())));
+
     KRunnerDialog::resizeEvent(event);
 }
 
@@ -243,11 +313,25 @@ Interface::~Interface()
     KRunnerSettings::setPastQueries(m_searchTerm->historyItems());
     KRunnerSettings::setQueryTextCompletionMode(m_searchTerm->completionMode());
     KRunnerSettings::self()->writeConfig();
+
+    // Before saving the size we resize to the default size, with the results container shown.
+    resize(m_defaultSize);
     KConfigGroup interfaceConfig(KGlobal::config(), "Interface");
     saveDialogSize(interfaceConfig);
     KGlobal::config()->sync();
 }
 
+
+void Interface::searchTermSetFocus()
+{
+    if (m_runnerManager->singleMode()) {
+        m_singleRunnerSearchTerm->setFocus();
+    } else {
+        m_searchTerm->setFocus();
+    }
+}
+
+
 void Interface::themeUpdated()
 {
     Plasma::Theme *theme = Plasma::Theme::defaultTheme();
@@ -257,7 +341,7 @@ void Interface::themeUpdated()
                                       .arg(buttonBgColor.red())
                                       .arg(buttonBgColor.green())
                                       .arg(buttonBgColor.blue())
-                                      .arg(theme->color(Plasma::Theme::HighlightColor).name(), "50%");
+                                      .arg(theme->color(Plasma::Theme::BackgroundColor).name(), "50%");
     buttonBgColor = theme->color(Plasma::Theme::TextColor);
     buttonStyleSheet += QString("QToolButton:hover { border: 2px solid %1; }")
                                .arg(theme->color(Plasma::Theme::HighlightColor).name());
@@ -265,17 +349,10 @@ void Interface::themeUpdated()
                                .arg(theme->color(Plasma::Theme::HighlightColor).name());
     m_configButton->setStyleSheet(buttonStyleSheet);
     m_activityButton->setStyleSheet(buttonStyleSheet);
+    m_helpButton->setStyleSheet(buttonStyleSheet);
     m_closeButton->setStyleSheet(buttonStyleSheet);
     //kDebug() << "stylesheet is" << buttonStyleSheet;
 
-    QPalette p = m_descriptionLabel->palette();
-    p.setColor(QPalette::WindowText, theme->color(Plasma::Theme::TextColor));
-    p.setColor(QPalette::Link, theme->color(Plasma::Theme::TextColor));
-    p.setColor(QPalette::LinkVisited, theme->color(Plasma::Theme::TextColor));
-    m_descriptionLabel->setPalette(p);
-    m_previousPage->setPalette(p);
-    m_nextPage->setPalette(p);
-
     //reset the icons
     m_configButton->setIcon(m_iconSvg->pixmap("configure"));
     m_activityButton->setIcon(m_iconSvg->pixmap("status"));
@@ -290,45 +367,28 @@ void Interface::clearHistory()
 
 void Interface::display(const QString &term)
 {
-    m_searchTerm->setFocus();
+    if (!term.isEmpty() || !isVisible() ||
+        m_runnerManager->singleMode() != m_singleRunnerIcon->isVisible()) {
+        resetInterface();
+    }
 
-    KWindowSystem::setOnDesktop(winId(), KWindowSystem::currentDesktop());
+    positionOnScreen();
+    searchTermSetFocus();
 
-    // TODO: set a nice welcome string when the string freeze lifts
-    m_descriptionLabel->clear();
-
-    show();
-    resetInterface();
-    centerOnScreen();
-    KWindowSystem::forceActiveWindow(winId());
-
-    if (!term.isEmpty()) {
+    if (m_runnerManager->singleMode()) {
+        if (term.isEmpty()) {
+            // We need to manually trigger queryTextEdited, since
+            // with an empty query it won't be triggered; still we need it
+            // to launch the query
+            queryTextEdited(QString());
+        } else {
+            m_singleRunnerSearchTerm->setText(term);
+        }
+    } else if (!term.isEmpty()) {
         m_searchTerm->setItemText(0, term);
     }
 }
 
-void Interface::centerOnScreen()
-{
-    int screen = 0;
-    if (QApplication::desktop()->numScreens() > 1) {
-        screen = QApplication::desktop()->screenNumber(QCursor::pos());
-    }
-
-    if (m_resultsView->isVisibleTo(this)) {
-        KDialog::centerOnScreen(this, screen);
-        return;
-    }
-
-    // center it as if the results view was already visible
-    QDesktopWidget *desktop = qApp->desktop();
-    QRect r = desktop->screenGeometry(screen);
-    int w = width();
-    int h = height() + m_resultsView->height();
-    move(r.left() + (r.width() / 2) - (w / 2),
-         r.top() + (r.height() / 2) - (h / 2));
-    //kDebug() << "moved to" << pos();
-}
-
 void Interface::setWidgetPalettes()
 {
     // a nice palette to use with the widgets
@@ -346,46 +406,82 @@ void Interface::resetInterface()
     setStaticQueryMode(false);
     m_delayedRun = false;
     m_searchTerm->setCurrentItem(QString(), true, 0);
-    m_descriptionLabel->clear();
-    m_descriptionLabel->hide();
-    m_previousPage->hide();
-    m_nextPage->hide();
+    m_singleRunnerSearchTerm->clear();
     m_resultsScene->clearQuery();
-    m_resultsView->hide();
-    m_dividerLine->hide();
-    setMinimumSize(QSize(MIN_WIDTH, 0));
-    //kDebug() << size() << minimumSizeHint();
-    resize(qMax(minimumSizeHint().width(), MIN_WIDTH), minimumSizeHint().height()+1);
+    resetResultsArea();
+    resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
+}
+
+void Interface::showHelp()
+{
+    QMap<QString, Plasma::QueryMatch> matches;
+    QList<Plasma::AbstractRunner*> runnerList;
+
+    Plasma::AbstractRunner *singleRunner = m_runnerManager->singleModeRunner();
+    if (singleRunner) {
+        runnerList << singleRunner;
+    } else {
+        runnerList = m_runnerManager->runners();
+    }
+
+
+    foreach (Plasma::AbstractRunner *runner, runnerList) {
+        int count = 0;
+        QIcon icon(runner->icon());
+        if (icon.isNull()) {
+            icon = KIcon("system-run");
+        }
+
+        foreach (const Plasma::RunnerSyntax &syntax, runner->syntaxes()) {
+            Plasma::QueryMatch match(0);
+            match.setType(Plasma::QueryMatch::InformationalMatch);
+            match.setIcon(icon);
+            match.setText(syntax.exampleQueriesWithTermDescription().join(", "));
+            match.setSubtext(syntax.description() + "\n" +
+                             i18n("(From %1, %2)", runner->name(), runner->description()));
+            match.setData(syntax.exampleQueries().first());
+            matches.insert(runner->name() + QString::number(++count), match);
+        }
+    }
+
+    m_resultsScene->setQueryMatches(matches.values());
+}
+
+void Interface::ensureVisibility(QGraphicsItem* item)
+{
+    bool oldState = m_resultsScene->itemsAcceptHoverEvents();
+    m_resultsScene->setItemsAcceptHoverEvents(false);
+    m_resultsView->ensureVisible(item,0,0);
+    m_resultsScene->setItemsAcceptHoverEvents(oldState);
 }
 
 void Interface::setStaticQueryMode(bool staticQuery)
 {
-    if (staticQuery) {
-        m_statusLayout->addWidget(m_closeButton);
-    } else {
-        m_buttonContainer->layout()->addWidget(m_closeButton);
+    // don't show the search and other control buttons in the case of a static querymatch
+    const bool visible = !staticQuery;
+    Plasma::AbstractRunner *singleRunner = m_runnerManager->singleModeRunner();
+    m_configButton->setVisible(visible && !singleRunner);
+    m_activityButton->setVisible(visible && !singleRunner);
+    m_helpButton->setVisible(visible);
+    m_searchTerm->setVisible(visible && !singleRunner);
+    m_singleRunnerSearchTerm->setVisible(visible && singleRunner);
+    if (singleRunner) {
+        m_singleRunnerIcon->setPixmap(singleRunner->icon().pixmap( QSize( 22, 22 )));
+        m_singleRunnerDisplayName->setText(singleRunner->name());
     }
-
-    // don't show the search and other control buttons in the case of a static query
-    m_buttonContainer->setVisible(!staticQuery);
-/*    m_configButton->setVisible(visible);
-    m_activityButton->setVisible(visible);
-    m_closeButton->setVisible(visible);
-    m_searchTerm->setVisible(visible);*/
+    m_singleRunnerIcon->setVisible(singleRunner);
+    m_singleRunnerDisplayName->setVisible(singleRunner);
 }
 
-void Interface::closeEvent(QCloseEvent *e)
+void Interface::hideEvent(QHideEvent *e)
 {
+    KRunnerDialog::hideEvent(e);
+
     if (!m_running) {
         resetInterface();
     } else {
         m_delayedRun = false;
-        m_resultsView->hide();
-        m_descriptionLabel->hide();
-        m_previousPage->hide();
-        m_nextPage->hide();
-        m_dividerLine->hide();
-        setMinimumSize(QSize(MIN_WIDTH, 0));
+        resetResultsArea();
         adjustSize();
     }
     e->accept();
@@ -400,14 +496,26 @@ void Interface::run(ResultItem *item)
 
     kDebug() << item->name() << item->id();
     m_delayedRun = false;
-    m_searchTerm->addToHistory(m_searchTerm->currentText());
 
     if (item->group() == Plasma::QueryMatch::InformationalMatch) {
         QString info = item->data();
+        int editPos = info.length();
 
         if (!info.isEmpty()) {
+            if (item->isQueryPrototype()) {
+                // lame way of checking to see if this is a Help Button generated match!
+                int index = info.indexOf(":q:");
+
+                if (index != -1) {
+                    editPos = index;
+                    info.replace(":q:", "");
+                }
+            }
+
             m_searchTerm->setItemText(0, info);
+            m_searchTerm->lineEdit()->setCursorPosition(editPos);
             m_searchTerm->setCurrentIndex(0);
+            QApplication::clipboard()->setText(info);
         }
         return;
     }
@@ -416,12 +524,22 @@ void Interface::run(ResultItem *item)
     close();
     m_resultsScene->run(item);
     m_running = false;
+
+    //TODO: check if run is succesful before adding the term to history
+    m_searchTerm->addToHistory(m_searchTerm->currentText().trimmed());
     resetInterface();
 }
 
+void Interface::actionTriggered()
+{
+    close();
+    resetInterface();
+}
+
+
 void Interface::runDefaultResultItem()
 {
-    if (m_queryRunning) {
+    if (m_queryRunning || m_delayedQueryTimer.isActive()) {
         m_delayedRun = true;
     } else {
         run(m_resultsScene->defaultResultItem());
@@ -432,26 +550,25 @@ void Interface::queryTextEdited(const QString &query)
 {
     m_delayedRun = false;
 
-    if (query.isEmpty()) {
+    if (query.isEmpty() && !m_runnerManager->singleMode()) {
+        m_delayedQueryTimer.stop();
         resetInterface();
         m_queryRunning = false;
     } else {
-        m_queryRunning = m_resultsScene->launchQuery(query) || m_queryRunning; //lazy OR?
+        m_delayedQueryTimer.start();
     }
 }
 
-void Interface::updateDescriptionLabel(ResultItem *item)
+void Interface::delayedQueryLaunch()
 {
-    // we want it always visible once we start showing it
-    // so that the interface isn't jumping all around
-    m_descriptionLabel->setVisible(m_resultsView->isVisible());
-    m_dividerLine->setVisible(m_resultsView->isVisible());
-    if (!item) {
-        m_descriptionLabel->setText(" ");
-    } else if (item->description().isEmpty()) {
-        m_descriptionLabel->setText(item->name());
-    } else {
-        m_descriptionLabel->setText(i18n("%1: %2",item->name() ,item->description()));
+    const QString query = (m_runnerManager->singleMode() ? m_singleRunnerSearchTerm->userText()
+                                                         : static_cast<KLineEdit*>(m_searchTerm->lineEdit())->userText());
+    QString runnerId;
+    if (m_runnerManager->singleMode()) {
+        runnerId = m_runnerManager->singleModeRunnerId();
+    }
+    if (!query.isEmpty() || m_runnerManager->singleMode()) {
+        m_queryRunning = m_resultsScene->launchQuery(query, runnerId) || m_queryRunning; //lazy OR?
     }
 }
 
@@ -460,9 +577,6 @@ void Interface::matchCountChanged(int count)
     m_queryRunning = false;
     bool show = count > 0;
     m_hideResultsTimer.stop();
-    bool pages = m_resultsScene->pageCount() > 1;
-    m_previousPage->setVisible(pages);
-    m_nextPage->setVisible(pages);
 
     if (show && m_delayedRun) {
         kDebug() << "delayed run with" << count << "items";
@@ -475,29 +589,43 @@ void Interface::matchCountChanged(int count)
     }
 
     if (show) {
-        //kDebug() << "showing!";
-        m_resultsView->show();
-        setMinimumSize(QSize(MIN_WIDTH, 0));
-        adjustSize();
+        //kDebug() << "showing!" << minimumSizeHint();
+
+        // Next 2 lines are a workaround to allow arrow
+        // keys navigation in krunner's result list.
+        // Patch submited in bugreport #211578
+        QEvent event(QEvent::WindowActivate);
+        QApplication::sendEvent(m_resultsView, &event);
+
+        resize(m_defaultSize);
+        m_resultsContainer->show();
+        m_resultsScene->resize(m_resultsView->width(), qMax(m_resultsView->height(), int(m_resultsScene->height())));
     } else {
         //kDebug() << "hiding ... eventually";
         m_delayedRun = false;
-        m_hideResultsTimer.start(2000);
+        m_hideResultsTimer.start(1000);
     }
 }
 
 void Interface::hideResultsArea()
 {
-    m_resultsView->hide();
-    m_descriptionLabel->hide();
-    m_descriptionLabel->clear();
-    m_previousPage->hide();
-    m_nextPage->hide();
-    m_dividerLine->hide();
-    m_searchTerm->setFocus();
-    setMinimumSize(QSize(MIN_WIDTH, 0));
-    adjustSize();
-    resize(minimumSizeHint());
+    searchTermSetFocus();
+
+    resetResultsArea();
+
+    resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
+}
+
+void Interface::resetResultsArea()
+{
+    setMinimumSize(QSize(MIN_WIDTH,0));
+    m_resultsContainer->hide();
+
+    //This is a workaround for some Qt bug which is not fully understood; it seems that
+    //adding a qgv to a layout and then hiding it gives some issues with resizing
+    //Calling updateGeometry should not be necessary, but it probably triggers some updates which do the trick
+
+    mainWidget()->updateGeometry();
 }
 
 #include "interface.moc"
