@@ -30,15 +30,18 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <ctype.h>
 
- 
 #include "Command.h"
 #include "ksysguardd.h"
 
 #include "diskstats.h"
 
 #define DISKSTATSBUFSIZE (32 * 1024)
-#define DISKDEVNAMELEN 16
+#define DISKDEVNAMELEN 20
+
+#define STRINGIFY(x) #x
+#define TOSTRING(x) STRINGIFY(x)
 
 typedef struct
 {
@@ -88,6 +91,16 @@ static int Dirty = 0;
 static void cleanup26DiskList( void );
 static int process26DiskIO( const char* buf );
 
+static void sanitize(char *str)  {
+    if(str == NULL)
+        return;
+    while (*str != 0)  {
+        if(*str == '\t' || *str == '\n' || *str == '\r' || *str == ' ' || !isascii(*str) )
+            *str = '?';
+        ++str;
+    }
+}
+
 void initDiskstats( struct SensorModul* sm ) {
 	char format[ 32 ];
 	char buf[ 1024 ];
@@ -151,12 +164,12 @@ void processDiskstats( void ) {
 
 	while (sscanf(iostatBufP, format, buf) == 1) {
 		buf[sizeof(buf) - 1] = '\0';
+        sanitize(buf);
 		iostatBufP += strlen(buf) + 1;  /* move IOstatBufP to next line */
-		
 		process26DiskIO(buf);
 	}
 
-	/* save exact time inverval between this and the last read of /proc/stat */
+	/* save exact time interval between this and the last read of /proc/stat */
 	timeInterval = currSampling.tv_sec - lastSampling.tv_sec +
 			( currSampling.tv_usec - lastSampling.tv_usec ) / 1000000.0;
 	lastSampling = currSampling;
@@ -226,7 +239,7 @@ static int process26DiskIO( const char* buf ) {
 		I/O completion time and the backlog that may be accumulating.
 	*/
 
-	switch (sscanf(buf, "%d %d %s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
+	switch (sscanf(buf, "%d %d %" TOSTRING(DISKDEVNAMELEN) "s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
 		&major, &minor, devname,
 		&rio, &rmrg, &rblk, &rtim,
 		&wio, &wmrg, &wblk, &wtim,
@@ -248,9 +261,10 @@ static int process26DiskIO( const char* buf ) {
 	
 		break;
 	default:
-		/* Something unexepected */
+		/* Something unexpected */
 		return -1;
 	}
+    devname[DISKDEVNAMELEN-1] = 0;
 	
 	last = 0;
 	ptr = DiskIO;
