@@ -43,6 +43,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "effects.h"
 #include "deleted.h"
 #include "paintredirector.h"
+#include "tabbox.h"
 
 #include <X11/extensions/shape.h>
 #include <QX11Info>
@@ -96,6 +97,7 @@ Client::Client( Workspace* ws )
     , delayedMoveResizeTimer( NULL )
     , in_group( NULL )
     , window_group( None )
+    , client_group( NULL )
     , in_layer( UnknownLayer )
     , ping_timer( NULL )
     , process_killer( NULL )
@@ -114,17 +116,20 @@ Client::Client( Workspace* ws )
     , border_right( 0 )
     , border_top( 0 )
     , border_bottom( 0 )
-    , sm_stacking_order( -1 )
-    , demandAttentionKNotifyTimer( NULL )
-    , paintRedirector( 0 )
     , padding_left( 0 )
     , padding_right( 0 )
     , padding_top( 0 )
     , padding_bottom( 0 )
+    , sm_stacking_order( -1 )
+    , demandAttentionKNotifyTimer( NULL )
+    , paintRedirector( 0 )
+    , electricMaximizing( false )
     { // TODO: Do all as initialization
 
     // Set the initial mapping state
     mapping_state = Withdrawn;
+    quick_tile_mode = QuickTileNone;
+    geom_pretile = QRect( 0, 0, 0, 0 );
     desk = 0; // No desktop yet
 
     mode = PositionCenter;
@@ -168,6 +173,10 @@ Client::Client( Workspace* ws )
 
     cmap = None;
 
+    // TabBoxClient
+    m_tabBoxClient = new TabBox::TabBoxClientImpl();
+    m_tabBoxClient->setClient( this );
+
     geom = QRect( 0, 0, 100, 100 ); // So that decorations don't start with size being (0,0)
     client_size = QSize( 100, 100 );
 #if defined(HAVE_XSYNC) || defined(HAVE_XDAMAGE)
@@ -194,6 +203,7 @@ Client::~Client()
     assert( block_geometry_updates == 0 );
     assert( !check_active_modal );
     delete bridge;
+    delete m_tabBoxClient;
     }
 
 // Use destroyClient() or releaseWindow(), Client instances cannot be deleted directly
@@ -235,6 +245,8 @@ void Client::releaseWindow( bool on_shutdown )
     XUnmapWindow( display(), frameId()); // Destroying decoration would cause ugly visual effect
     destroyDecoration();
     cleanGrouping();
+    if( clientGroup() )
+        clientGroup()->remove( this, QRect(), true );
     if( !on_shutdown )
         {
         workspace()->removeClient( this, Allowed );
@@ -297,6 +309,8 @@ void Client::destroyClient()
     workspace()->clientHidden( this );
     destroyDecoration();
     cleanGrouping();
+    if( clientGroup() )
+        clientGroup()->remove( this, QRect(), true );
     workspace()->removeClient( this, Allowed );
     client = None; // invalidate
     XDestroyWindow( display(), wrapper );
@@ -431,13 +445,27 @@ void Client::layoutDecorationRects(QRect &left, QRect &top, QRect &right, QRect
     if (mode == WindowRelative)
         r.translate(-padding_left, -padding_top);
 
-    top = QRect(r.x(), r.y(), r.width(), padding_top + border_top);
-    bottom = QRect(r.x(), r.y() + r.height() - padding_bottom - border_bottom,
-                   r.width(), padding_bottom + border_bottom);
+    NETStrut strut = info->frameOverlap();
+
+    // Ignore the overlap strut when compositing is disabled
+    if (!compositing() || !Workspace::self()->decorationSupportsFrameOverlap())
+        strut.left = strut.top = strut.right = strut.bottom = 0;
+    else if (strut.left == -1 && strut.top == -1 && strut.right == -1 && strut.bottom == -1)
+        {
+        top = QRect(r.x(), r.y(), r.width(), r.height() / 3);
+        left = QRect(r.x(), r.y() + top.height(), width() / 2, r.height() / 3);
+        right = QRect(r.x() + left.width(), r.y() + top.height(), r.width() - left.width(), left.height());
+        bottom = QRect(r.x(), r.y() + top.height() + left.height(), r.width(), r.height() - left.height() - top.height());
+        return;
+        }
+
+    top = QRect(r.x(), r.y(), r.width(), padding_top + border_top + strut.top);
+    bottom = QRect(r.x(), r.y() + r.height() - padding_bottom - border_bottom - strut.bottom,
+                   r.width(), padding_bottom + border_bottom + strut.bottom);
     left = QRect(r.x(), r.y() + top.height(),
-                 padding_left + border_left, r.height() - top.height() - bottom.height());
-    right = QRect(r.x() + r.width() - padding_right - border_right, r.y() + top.height(),
-                  padding_right + border_right, r.height() - top.height() - bottom.height());
+                 padding_left + border_left + strut.left, r.height() - top.height() - bottom.height());
+    right = QRect(r.x() + r.width() - padding_right - border_right - strut.right, r.y() + top.height(),
+                  padding_right + border_right + strut.right, r.height() - top.height() - bottom.height());
     }
 
 QRegion Client::decorationPendingRegion() const
@@ -449,7 +477,7 @@ QRegion Client::decorationPendingRegion() const
 
 void Client::repaintDecorationPending()
     {
-    if ( compositing() )
+    if (compositing()) 
         {
         // The scene will update the decoration pixmaps in the next painting pass
         // if it has not been already repainted before
@@ -567,6 +595,26 @@ void Client::resizeDecorationPixmaps()
     triggerDecorationRepaint();
     }
 
+QRect Client::transparentRect() const
+    {
+    if (isShade())
+        return QRect();
+
+    NETStrut strut = info->frameOverlap();
+    // Ignore the strut when compositing is disabled or the decoration doesn't support it
+    if (!compositing() || !Workspace::self()->decorationSupportsFrameOverlap())
+        strut.left = strut.top = strut.right = strut.bottom = 0;
+    else if (strut.left == -1 && strut.top == -1 && strut.right == -1 && strut.bottom == -1)
+        return QRect();
+
+    const QRect r = QRect(clientPos(), clientSize())
+                        .adjusted(strut.left, strut.top, -strut.right, -strut.bottom);
+    if (r.isValid())
+        return r;
+
+    return QRect();
+    }
+
 void Client::detectNoBorder()
     {
     if( shape())
@@ -647,7 +695,7 @@ bool Client::noBorder() const
 
 bool Client::userCanSetNoBorder() const
     {
-    return !isFullScreen() && !isShade();
+    return !isFullScreen() && !isShade() && ( clientGroup() == NULL || !(clientGroup()->items().count() > 1));
     }
 
 void Client::setNoBorder( bool set )
@@ -840,6 +888,10 @@ void Client::minimize( bool avoid_animation )
     workspace()->updateFocusChains( this, Workspace::FocusChainMakeLast );
     if( effects && !avoid_animation ) // TODO: Shouldn't it tell effects at least about the change?
         static_cast<EffectsHandlerImpl*>(effects)->windowMinimized( effectWindow());
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 void Client::unminimize( bool avoid_animation )
@@ -855,6 +907,10 @@ void Client::unminimize( bool avoid_animation )
     updateWindowRules();
     if( effects && !avoid_animation )
         static_cast<EffectsHandlerImpl*>( effects )->windowUnminimized( effectWindow() );
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 QRect Client::iconGeometry() const
@@ -958,6 +1014,10 @@ void Client::setShade( ShadeMode mode )
     workspace()->updateMinimizedOfTransients( this );
     decoration->shadeChange();
     updateWindowRules();
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 void Client::shadeHover()
@@ -987,7 +1047,7 @@ void Client::updateVisibility()
     {
     if( deleting )
         return;
-    if( hidden )
+    if( hidden && ( clientGroup() == NULL || clientGroup()->visible() == this ))
         {
         info->setState( NET::Hidden, NET::Hidden );
         setSkipTaskbar( true, false ); // Also hide from taskbar
@@ -997,7 +1057,8 @@ void Client::updateVisibility()
             internalHide( Allowed );
         return;
         }
-    setSkipTaskbar( original_skip_taskbar, false ); // Reset from 'hidden'
+    if( clientGroup() == NULL || clientGroup()->visible() == this )
+        setSkipTaskbar( original_skip_taskbar, false ); // Reset from 'hidden'
     if( minimized )
         {
         info->setState( NET::Hidden, NET::Hidden );
@@ -1387,6 +1448,10 @@ void Client::setDesktop( int desktop )
     workspace()->updateFocusChains( this, Workspace::FocusChainMakeFirst );
     updateVisibility();
     updateWindowRules();
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 /**
@@ -1409,6 +1474,10 @@ void Client::setOnAllDesktops( bool b )
         setDesktop( NET::OnAllDesktops );
     else
         setDesktop( workspace()->currentDesktop());
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 /**
@@ -1559,7 +1628,11 @@ void Client::setCaption( const QString& _s, bool force )
             info->setVisibleIconName( ( cap_iconic + cap_suffix ).toUtf8() );
 
         if( isManaged() && decoration != NULL )
+            {
+            if (client_group)
+                client_group->updateItems();
             decoration->captionChange();
+            }
         }
     }
 
@@ -1597,6 +1670,43 @@ QString Client::caption( bool full ) const
     return full ? cap_normal + cap_suffix : cap_normal;
     }
 
+void Client::dontMoveResize()
+    {
+    buttonDown = false;
+    stopDelayedMoveResize();
+    if( moveResizeMode )
+        finishMoveResize( false );
+    }
+
+void Client::setClientShown( bool shown )
+    {
+    if( deleting )
+        return; // Don't change shown status if this client is being deleted
+    if( shown && hidden )
+        {
+        map( Allowed );
+        hidden = false;
+        //updateVisibility();
+        //updateAllowedActions();
+        if( options->inactiveTabsSkipTaskbar )
+            setSkipTaskbar( false, false );
+        takeFocus( Allowed );
+        autoRaise();
+        workspace()->updateFocusChains( this, Workspace::FocusChainMakeFirst );
+        }
+    if( !shown && !hidden )
+        {
+        unmap( Allowed );
+        hidden = true;
+        //updateVisibility();
+        //updateAllowedActions();
+        if( options->inactiveTabsSkipTaskbar )
+            setSkipTaskbar( true, false ); // TODO: Causes reshuffle of the taskbar
+        workspace()->updateFocusChains( this, Workspace::FocusChainMakeLast );
+        addWorkspaceRepaint( visibleRect() );
+        }
+    }
+
 void Client::getWMHints()
     {
     XWMHints* hints = XGetWMHints( display(), window() );
@@ -1703,7 +1813,7 @@ void Client::getWindowProtocols()
 
     if( XGetWMProtocols( display(), window(), &p, &n ))
         {
-        for( i = 0; i < n; i++ )
+        for( i = 0; i < n; ++i )
             {
             if( p[i] == atoms->wm_delete_window )
                 Pdeletewindow = 1;
@@ -1755,6 +1865,9 @@ void Client::getSyncCounter()
                 &attrs );
             }
         }
+
+    if( ret == Success )
+        XFree( propRet );
 #endif
 }
 
