@@ -24,7 +24,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kwineffects.h>
 
 #include <QHash>
-#include <QPixmap>
 #include <QRect>
 #include <QRegion>
 #include <QSize>
@@ -34,6 +33,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kwinglutils.h>
 #include <kwinxrenderutils.h>
 
+#include "boxswitch_proxy.h"
+
 namespace KWin
 {
 
@@ -57,6 +58,9 @@ class BoxSwitchEffect
         virtual void tabBoxClosed();
         virtual void tabBoxUpdated();
         virtual void windowClosed( EffectWindow* w );
+        virtual void* proxy();
+        void activateFromProxy( int mode, bool animate, bool showText, float positioningFactor );
+        void paintWindowsBox( const QRegion& region );
     private:
         class ItemInfo;
         void setActive();
@@ -113,6 +117,15 @@ class BoxSwitchEffect
         QQueue<Direction> scheduled_directions;
         EffectWindow* edge_window;
         EffectWindow* right_window;
+
+        bool primaryTabBox;
+        bool secondaryTabBox;
+
+        BoxSwitchEffectProxy mProxy;
+        bool mProxyActivated;
+        bool mProxyAnimateSwitch;
+        bool mProxyShowText;
+        float mPositioningFactor;
     };
 
 class BoxSwitchEffect::ItemInfo
