@@ -91,6 +91,7 @@ bool Client::manage( Window w, bool isMapped )
         NET::WM2ExtendedStrut |
         NET::WM2Opacity |
         NET::WM2FullscreenMonitors |
+        NET::WM2FrameOverlap |
         0;
 
     info = new WinInfo( this, display(), client, rootWindow(), properties, 2 );
@@ -296,6 +297,42 @@ bool Client::manage( Window w, bool isMapped )
     if( placementDone )
         move( geom.x(), geom.y() ); // Before gravitating
 
+    // Create client group if the window will have a decoration
+    bool dontKeepInArea = false;
+    if( !noBorder() )
+        {
+        client_group = NULL;
+        // Automatically add to previous groups on session restore
+        if( session && session->clientGroupClient && session->clientGroupClient != this )
+            session->clientGroupClient->clientGroup()->add( this, -1, true );
+        else if( isMapped )
+            // If the window is already mapped (Restarted KWin) add any windows that already have the
+            // same geometry to the same client group. (May incorrectly handle maximized windows)
+            foreach( ClientGroup* group, workspace()->clientGroups )
+                if( geom == QRect( group->visible()->pos(), group->visible()->clientSize() ) &&
+                    desk == group->visible()->desktop() &&
+                    group->visible()->maximizeMode() != MaximizeFull )
+                    {
+                    group->add( this, -1, true );
+                    break;
+                    }
+        if( !client_group && !isMapped && !session )
+            { // Attempt to automatically group similar windows
+            const Client* similar = workspace()->findSimilarClient( this );
+            if( similar && similar->clientGroup() && !similar->noBorder() )
+                {
+                similar->clientGroup()->add( this, -1,
+                    rules()->checkAutogroupInForeground( options->autogroupInForeground ));
+                // Don't move entire group
+                geom = QRect( similar->pos() + similar->clientPos(), similar->clientSize() );
+                placementDone = true;
+                dontKeepInArea = true;
+                }
+            }
+        if( !client_group )
+            client_group = new ClientGroup( this );
+        }
+
     updateDecoration( false ); // Also gravitates
     // TODO: Is CentralGravity right here, when resizing is done after gravitating?
     plainResize( rules()->checkSize( sizeForClientSize( geom.size() ), !isMapped ));
@@ -315,7 +352,7 @@ bool Client::manage( Window w, bool isMapped )
         placementDone = true;
         }
 
-    if(( !isSpecialWindow() || isToolbar() ) && isMovable() )
+    if(( !isSpecialWindow() || isToolbar() ) && isMovable() && !dontKeepInArea )
         keepInArea( area, partial_keep_in_area );
 
     updateShape();
@@ -354,6 +391,7 @@ bool Client::manage( Window w, bool isMapped )
     if( init_minimize )
         minimize( true ); // No animation
 
+
     // SELI TODO: This seems to be mainly for kstart and ksystraycmd
     // probably should be replaced by something better
     bool doNotShow = false;
@@ -370,6 +408,7 @@ bool Client::manage( Window w, bool isMapped )
         setSkipTaskbar( session->skipTaskbar, true );
         setSkipPager( session->skipPager );
         setShade( session->shaded ? ShadeNormal : ShadeNone );
+        setOpacity( session->opacity );
         if( session->maximized != MaximizeRestore )
             {
             maximize( MaximizeMode( session->maximized ));
