@@ -71,6 +71,7 @@ QString KSysGuard::Process::translatedStatus() const {
         case Zombie: return i18nc("process status", "zombie");
         case Stopped: return i18nc("process status", "stopped");
         case Paging: return i18nc("process status", "paging");
+//        case Ended: return i18nc("process status", "finished");
         default: return i18nc("process status", "unknown");
     }
 }
@@ -109,6 +110,19 @@ void KSysGuard::Process::clear() {
     ioPriorityClass = None;
     ioniceLevel = -1;
     scheduler = Other;
+    ioCharactersRead = 0;
+    ioCharactersWritten = 0;
+    ioReadSyscalls = 0;
+    ioWriteSyscalls = 0;
+    ioCharactersActuallyRead = 0;
+    ioCharactersActuallyWritten = 0;
+    ioCharactersReadRate = 0;
+    ioCharactersWrittenRate = 0;
+    ioReadSyscallsRate = 0;
+    ioWriteSyscallsRate = 0;
+    ioCharactersActuallyReadRate = 0;
+    ioCharactersActuallyWrittenRate = 0;
+
     changes = Process::Nothing;
 }
 void KSysGuard::Process::setLogin(QString _login) {
@@ -244,4 +258,64 @@ void KSysGuard::Process::setStatus(ProcessStatus _status) {
     status = _status;
     changes |= Process::Status;
 } 
+void KSysGuard::Process::setIoCharactersRead(qlonglong number) {
+    if(number == ioCharactersRead) return;
+    ioCharactersRead = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoCharactersWritten(qlonglong number) {
+    if(number == ioCharactersWritten) return;
+    ioCharactersWritten = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoReadSyscalls(qlonglong number) {
+    if(number == ioReadSyscalls) return;
+    ioReadSyscalls = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoWriteSyscalls(qlonglong number) {
+    if(number == ioWriteSyscalls) return;
+    ioWriteSyscalls = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoCharactersActuallyRead(qlonglong number) {
+    if(number == ioCharactersActuallyRead) return;
+    ioCharactersActuallyRead = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoCharactersActuallyWritten(qlonglong number) {
+    if(number == ioCharactersActuallyWritten) return;
+    ioCharactersActuallyWritten = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoCharactersReadRate(long number) {
+    if(number == ioCharactersReadRate) return;
+    ioCharactersReadRate = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoCharactersWrittenRate(long number) {
+    if(number == ioCharactersWrittenRate) return;
+    ioCharactersWrittenRate = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoReadSyscallsRate(long number) {
+    if(number == ioReadSyscallsRate) return;
+    ioReadSyscallsRate = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoWriteSyscallsRate(long number) {
+    if(number == ioWriteSyscallsRate) return;
+    ioWriteSyscallsRate = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoCharactersActuallyReadRate(long number) {
+    if(number == ioCharactersActuallyReadRate) return;
+    ioCharactersActuallyReadRate = number;
+    changes |= Process::IO;
+}
+void KSysGuard::Process::setIoCharactersActuallyWrittenRate(long number) {
+    if(number == ioCharactersActuallyWrittenRate) return;
+    ioCharactersActuallyWrittenRate = number;
+    changes |= Process::IO;
+}
 
