@@ -35,7 +35,7 @@ namespace KSysGuard
 
   class KDE_EXPORT Process {
     public:
-    enum ProcessStatus { Running, Sleeping, DiskSleep, Zombie, Stopped, Paging, OtherStatus };
+    enum ProcessStatus { Running, Sleeping, DiskSleep, Zombie, Stopped, Paging, Ended, OtherStatus=99 };
     enum IoPriorityClass { None, RealTime, BestEffort, Idle };
     enum Scheduler { Other = 0, Fifo, RoundRobin, Batch };
     Process();
@@ -82,6 +82,20 @@ namespace KSysGuard
     void setCommand(QString command); ///< The command the process was launched with
     void setStatus( ProcessStatus status); ///< Whether the process is running/sleeping/etc
 
+    void setIoCharactersRead(qlonglong number); ///< The number of bytes which this task has caused to be read from storage
+    void setIoCharactersWritten(qlonglong number); ///< The number of bytes which this task has caused, or shall cause to be written to disk. 
+    void setIoReadSyscalls(qlonglong number); ///< Number of read I/O operations, i.e. syscalls like read() and pread().
+    void setIoWriteSyscalls(qlonglong number); ///< Number of write I/O operations, i.e. syscalls like write() and pwrite().
+    void setIoCharactersActuallyRead(qlonglong number); ///< Number of bytes which this process really did cause to be fetched from the storage layer.
+    void setIoCharactersActuallyWritten(qlonglong number); ///< Attempt to count the number of bytes which this process caused to be sent to the storage layer.
+
+    void setIoCharactersReadRate(long number); ///< The rate, in bytes per second, which this task has caused to be read from storage
+    void setIoCharactersWrittenRate(long number); ///< The rate, in bytes per second, which this task has caused, or shall cause to be written to disk. 
+    void setIoReadSyscallsRate(long number); ///< Number of read I/O operations per second, i.e. syscalls like read() and pread().
+    void setIoWriteSyscallsRate(long number); ///< Number of write I/O operations per second, i.e. syscalls like write() and pwrite().
+    void setIoCharactersActuallyReadRate(long number); ///< Number of bytes per second which this process really did cause to be fetched from the storage layer.
+    void setIoCharactersActuallyWrittenRate(long number); ///< Attempt to count the number of bytes per second which this process caused to be sent to the storage layer.
+
     /* The member variables are made to public for efficiency, but should only be read from. */
     QString login; 
     qlonglong uid; 
@@ -94,25 +108,38 @@ namespace KSysGuard
     qlonglong sgid; 
     qlonglong fsgid; 
 
-    qlonglong tracerpid; 
-    QByteArray tty; 
-    qlonglong userTime; 
-    qlonglong sysTime;  
-    int userUsage; 
-    int sysUsage;  
-    int totalUserUsage; 
-    int totalSysUsage; 
-    unsigned long numChildren; 
-    int niceLevel;      
-    Scheduler scheduler; 
-    IoPriorityClass ioPriorityClass; 
-    int ioniceLevel;    
-    qlonglong vmSize;   
-    qlonglong vmRSS;    
-    qlonglong vmURSS;   
-    QString name;  
-    QString command; 
-    ProcessStatus status; 
+    qlonglong tracerpid;
+    QByteArray tty;
+    qlonglong userTime;
+    qlonglong sysTime;
+    int userUsage;
+    int sysUsage;
+    int totalUserUsage;
+    int totalSysUsage;
+    unsigned long numChildren;
+    int niceLevel;
+    Scheduler scheduler;
+    IoPriorityClass ioPriorityClass;
+    int ioniceLevel;
+    qlonglong vmSize;
+    qlonglong vmRSS;
+    qlonglong vmURSS;
+    QString name;
+    QString command;
+    ProcessStatus status;
+    qlonglong ioCharactersRead;
+    qlonglong ioCharactersWritten;
+    qlonglong ioReadSyscalls;
+    qlonglong ioWriteSyscalls;
+    qlonglong ioCharactersActuallyRead;
+    qlonglong ioCharactersActuallyWritten;
+
+    long ioCharactersReadRate;
+    long ioCharactersWrittenRate;
+    long ioReadSyscallsRate;
+    long ioWriteSyscallsRate;
+    long ioCharactersActuallyReadRate;
+    long ioCharactersActuallyWrittenRate;
 
     QList<Process *> children;  ///< A list of all the direct children that the process has.  Children of children are not listed here, so note that children_pids <= numChildren
     QTime timeKillWasSent; ///< This is usually a NULL time.  When trying to kill a process, this is the time that the kill signal was sent to the process.
@@ -143,13 +170,14 @@ namespace KSysGuard
         Name = 0x400,
         Command = 0x800,
         Status = 0x1000,
-        Login = 0x2000
+        Login = 0x2000,
+        IO = 0x4000
     };
     Q_DECLARE_FLAGS(Changes, Change)
 
     Changes changes;  /**< A QFlags representing what has changed */
 
-    /** This is the number of 1/10ths of a second since this 
+    /** This is the number of 1/1000ths of a second since this
      *  particular process was last updated compared to when all the processes
      *  were updated. The purpose is to allow a more fine tracking of the time
      *  a process has been running for. 
