@@ -2,6 +2,7 @@
  *   Copyright (C) 2007-2008 by Riccardo Iaconelli <riccardo@kde.org>      *
  *   Copyright (C) 2007-2008 by Sebastian Kuegler <sebas@kde.org>          *
  *   Copyright (C) 2008-2009 by Davide Bettio <davide.bettio@kdemail.net>  *
+ *   Copyright (C) 2009 by John Layt <john@layt.net>                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -45,7 +46,10 @@
 #include <KDebug>
 #include <KDialog>
 #include <KGlobalSettings>
+#include <KRun>
 #include <KLocale>
+#include <KService>
+#include <KServiceTypeTrader>
 #include <KTimeZone>
 #include <KToolInvocation>
 #include <KMessageBox>
@@ -72,9 +76,10 @@ public:
         : q(clockapplet),
           timezone(ClockApplet::localTimezoneUntranslated()),
           clipboardMenu(0),
+          adjustSystemTimeAction(0),
           label(0),
-          forceTzDisplay(false),
-          displayHolidays(false)
+          calendarWidget(0),
+          forceTzDisplay(false)
     {}
 
     ClockApplet *q;
@@ -85,16 +90,16 @@ public:
     QPoint clicked;
     QStringList selectedTimezones;
     KMenu *clipboardMenu;
+    QAction *adjustSystemTimeAction;
     QString prettyTimezone;
     Plasma::Label *label;
-    QString holidaysRegion;
+    Plasma::Calendar *calendarWidget;
     int announceInterval;
     int prevHour;
     int prevMinute;
     bool forceTzDisplay : 1;
-    bool displayHolidays : 1;
 
-    void addTzToTipText(QString &subText, QString tz)
+    void addTzToTipText(QString &subText, const QString& tz)
     {
         Plasma::DataEngine::Data data = q->dataEngine("time")->query(tz);
 
@@ -102,16 +107,16 @@ public:
             subText.append("<br><b>UTC</b>&nbsp;");
         } else {
             subText.append("<br><b>")
-                   .append(data["Timezone City"].toString().replace("_", "&nbsp;"))
+                   .append(data["Timezone City"].toString().replace('_', "&nbsp;"))
                    .append("</b> ");
         }
 
-        subText.append(KGlobal::locale()->formatTime(data["Time"].toTime(), false).replace(" ", "&nbsp;"))
+        subText.append(KGlobal::locale()->formatTime(data["Time"].toTime(), false).replace(' ', "&nbsp;"))
                .append(",&nbsp;")
-               .append(KGlobal::locale()->formatDate(data["Date"].toDate()).replace(" ", "&nbsp;"));
+               .append(q->calendar()->formatDate(data["Date"].toDate()).replace(' ', "&nbsp;"));
     }
 
-    void createCalendar()
+    void createCalendarExtender()
     {
         if (!q->extender()->hasItem("calendar")) {
             Plasma::ExtenderItem *eItem = new Plasma::ExtenderItem(q->extender());
@@ -122,7 +127,7 @@ public:
 
     void createToday()
     {
-        QString tmpStr = "isHoliday:" + holidaysRegion + ":" + QDate::currentDate().toString(Qt::ISODate);
+        QString tmpStr = "isHoliday:" + calendarWidget->holidaysRegion() + ':' + QDate::currentDate().toString(Qt::ISODate);
         bool isHoliday = q->dataEngine("calendar")->query(tmpStr).value(tmpStr).toBool();
 
         Plasma::ExtenderItem *todayExtender = q->extender()->item("today");
@@ -137,7 +142,7 @@ public:
         }
     }
 
-    void createDateExtender(QDate date)
+    void createDateExtender(const QDate& date)
     {
         Plasma::ExtenderItem *eItem = new Plasma::ExtenderItem(q->extender());
         eItem->setName("dateExtender-" + date.toString(Qt::ISODate));
@@ -149,7 +154,7 @@ public:
         QList<Plasma::ExtenderItem *> extenders = q->extender()->items();
         for (int i = 0; i < extenders.size(); i++){
             Plasma::ExtenderItem *eItem = extenders.at(i);
-            if (eItem->name().startsWith("dateExtender-") && !eItem->isDetached()){
+            if (eItem->name().startsWith(QLatin1String("dateExtender-")) && !eItem->isDetached()){
                 eItem->destroy();
             }
         }
@@ -161,11 +166,11 @@ public:
         if (timezone == "UTC")  {
             prettyTimezone = timezonetranslated;
         } else if (!q->isLocalTimezone()) {
-            QStringList tzParts = timezonetranslated.split("/", QString::SkipEmptyParts);
+            QStringList tzParts = timezonetranslated.split('/', QString::SkipEmptyParts);
             if (tzParts.count() == 1) {
                 prettyTimezone = timezonetranslated;
             } else {
-                prettyTimezone = tzParts.value(1);
+                prettyTimezone = tzParts.last();
             }
         } else {
             prettyTimezone = localTimezone();
@@ -173,17 +178,6 @@ public:
 
         prettyTimezone = prettyTimezone.replace('_', ' ');
     }
-
-    Plasma::Calendar *calendarWidget()
-    {
-        Plasma::ExtenderItem *item = q->extender()->item("calendar");
-        if (!item) {
-            return 0;
-        }
-
-        return dynamic_cast<Plasma::Calendar *>(item->widget());
-    }
-
 };
 
 ClockApplet::ClockApplet(QObject *parent, const QVariantList &args)
@@ -220,6 +214,7 @@ void ClockApplet::speakTime(const QTime &time)
         }
 
         QDBusInterface ktts("org.kde.kttsd", "/KSpeech", "org.kde.KSpeech");
+        ktts.asyncCall("setApplicationName", "plasmaclock");
         QString text;
         if (time.minute() == 0) {
             if (KGlobal::locale()->use12Clock()) {
@@ -281,14 +276,13 @@ void ClockApplet::updateTipContent()
 {
     Plasma::ToolTipContent tipData;
 
-    {
-        // the main text contains the current timezone's time and date
-        Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
-        QString mainText = d->prettyTimezone + " ";
-        mainText += KGlobal::locale()->formatTime(data["Time"].toTime(), false) + "<br>";
-        mainText += KGlobal::locale()->formatDate(data["Date"].toDate());
-        tipData.setMainText(mainText);
-    }
+    // the main text contains the current timezone's time and date
+    Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
+    QString mainText = d->prettyTimezone + ' ';
+    mainText += KGlobal::locale()->formatTime(data["Time"].toTime(), false) + "<br>";
+    QDate tipDate = data["Date"].toDate();
+    mainText += calendar()->formatDate(tipDate);
+    tipData.setMainText(mainText);
 
     QString subText;
     if (!isLocalTimezone()) {
@@ -303,12 +297,22 @@ void ClockApplet::updateTipContent()
         d->addTzToTipText(subText, tz);
     }
 
+    QString property = d->calendarWidget->dateProperty(tipDate);
+    if (!property.isEmpty()) {
+        QString countryString = KGlobal::locale()->countryCodeToName(d->calendarWidget->holidaysRegion());
+        if (countryString.isEmpty()) {
+            subText.append("<br>").append(property);
+        } else {
+            subText.append("<br><b>").append(countryString).append("</b> ").append(property);
+        }
+    }
+
     tipData.setSubText(subText);
 
     // query for custom content
     Plasma::ToolTipContent customContent = toolTipContent();
     if (customContent.image().isNull()) {
-        tipData.setImage(KIcon("chronometer").pixmap(IconSize(KIconLoader::Desktop)));
+        tipData.setImage(KIcon(icon()).pixmap(IconSize(KIconLoader::Desktop)));
     } else {
         tipData.setImage(customContent.image());
     }
@@ -332,55 +336,17 @@ Plasma::ToolTipContent ClockApplet::toolTipContent()
     return Plasma::ToolTipContent();
 }
 
-bool caseInsensitiveLessThan(const QString &s1, const QString &s2)
-{
-    return s1.toLower() < s2.toLower();
-}
-
 void ClockApplet::createConfigurationInterface(KConfigDialog *parent)
 {
     createClockConfigurationInterface(parent);
 
     QWidget *generalWidget = new QWidget();
     d->generalUi.setupUi(generalWidget);
-
-    parent->addPage(generalWidget, i18n("General"), Applet::icon());
-
-    d->generalUi.displayHolidays->setChecked(d->displayHolidays);
-    QStringList regions = dataEngine("calendar")->query("holidaysRegions").value("holidaysRegions").toStringList();
-    QMap<QString, QPair<QString, QString> > names;
-    foreach (const QString &region, regions) {
-        if (region.isEmpty()) {
-            continue;
-        }
-
-        // get a proper name!
-        QString name = KGlobal::locale()->countryCodeToName(region);
-
-        if (name.isEmpty()) {
-            name = region;
-            name[0] = name.at(0).toUpper();
-        }
-
-        names.insert(name.toLower(), qMakePair(name, region));
-    }
-
-    int i = 0;
-
-    QMapIterator<QString, QPair<QString, QString> > it(names);
-    while (it.hasNext()) {
-        it.next();
-        QString name = it.value().first;
-        QString region = it.value().second;
-        d->generalUi.regionComboBox->addItem(name, region);
-        if (region == d->holidaysRegion) {
-            d->generalUi.regionComboBox->setCurrentIndex(i);
-        }
-        ++i;
-    }
-
+    parent->addPage(generalWidget, i18nc("General configuration page", "General"), Applet::icon());
     d->generalUi.interval->setValue(d->announceInterval);
 
+    d->calendarWidget->createConfigurationInterface(parent);
+
     QWidget *widget = new QWidget();
     d->ui.setupUi(widget);
     d->ui.searchLine->addTreeWidget(d->ui.timeZones);
@@ -402,17 +368,7 @@ void ClockApplet::createConfigurationInterface(KConfigDialog *parent)
     parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
-    connect(d->ui.timeZones, SIGNAL(itemSelectionChanged()), this, SLOT(updateClockDefaultsTo()));
-
-#if 0
-#ifdef CLOCK_APPLET_CONF
-    ui.localTimeZone->setChecked(isLocalTimezone());
-    ui.timeZones->setEnabled(!isLocalTimezone());
-    foreach (const QString &str, selectedTimezones) {
-        ui.timeZones->setSelected(str, true);
-    }
-#endif
-#endif
+    connect(d->ui.timeZones, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(updateClockDefaultsTo()));
 }
 
 void ClockApplet::createClockConfigurationInterface(KConfigDialog *parent)
@@ -444,15 +400,7 @@ void ClockApplet::configAccepted()
     setCurrentTimezone(d->defaultTimezone);
     changeEngineTimezone(cur, d->defaultTimezone);
 
-    d->displayHolidays = d->generalUi.displayHolidays->isChecked();
-    cg.writeEntry("displayHolidays", d->displayHolidays);
-    d->holidaysRegion = d->generalUi.regionComboBox->itemData(d->generalUi.regionComboBox->currentIndex()).toString();
-    cg.writeEntry("holidaysRegion", d->holidaysRegion);
-
-    Plasma::Calendar *calendar = d->calendarWidget();
-    if (calendar) {
-        calendar->setRegion(d->displayHolidays ? d->holidaysRegion : QString());
-    }
+    d->calendarWidget->configAccepted(cg);
 
     d->announceInterval = d->generalUi.interval->value();
     cg.writeEntry("announceInterval", d->announceInterval);
@@ -469,16 +417,16 @@ void ClockApplet::updateClockDefaultsTo()
     QString oldSelection = d->ui.clockDefaultsTo->currentText();
     d->ui.clockDefaultsTo->clear();
     d->ui.clockDefaultsTo->addItem(localTimezone(), localTimezone());
-    foreach(const QString &tz, d->ui.timeZones->selection())
+    foreach(const QString &tz, d->ui.timeZones->selection()) {
         d->ui.clockDefaultsTo->addItem(KTimeZoneWidget::displayName(KTimeZone(tz)), tz);
+    }
     int newPosition = d->ui.clockDefaultsTo->findText(oldSelection);
     if (newPosition >= 0) {
         d->ui.clockDefaultsTo->setCurrentIndex(newPosition);
     }
     if (d->ui.clockDefaultsTo->count() > 1) {
         d->ui.clockDefaultsTo->setEnabled(true);
-    }
-    else {
+    } else {
         // Only "Local" in ui.clockDefaultsTo
         d->ui.clockDefaultsTo->setEnabled(false);
     }
@@ -503,13 +451,38 @@ QList<QAction *> ClockApplet::contextualActions()
         d->clipboardMenu->setIcon(KIcon("edit-copy"));
         connect(d->clipboardMenu, SIGNAL(aboutToShow()), this, SLOT(updateClipboardMenu()));
         connect(d->clipboardMenu, SIGNAL(triggered(QAction*)), this, SLOT(copyToClipboard(QAction*)));
+
+        KService::List offers = KServiceTypeTrader::self()->query("KCModule", "Library == 'kcm_clock'");
+        if (!offers.isEmpty() && hasAuthorization("LaunchApp")) {
+            d->adjustSystemTimeAction = new QAction(this);
+            d->adjustSystemTimeAction->setText(i18n("Adjust Date and Time"));
+            d->adjustSystemTimeAction->setIcon(KIcon("preferences-system-time"));
+            connect(d->adjustSystemTimeAction, SIGNAL(triggered()), this, SLOT(launchTimeControlPanel()));
+        }
     }
 
     QList<QAction*> contextualActions;
     contextualActions << d->clipboardMenu->menuAction();
+
+    if (d->adjustSystemTimeAction) {
+        contextualActions << d->adjustSystemTimeAction;
+    }
     return contextualActions;
 }
 
+void ClockApplet::launchTimeControlPanel()
+{
+    KService::List offers = KServiceTypeTrader::self()->query("KCModule", "Library == 'kcm_clock'");
+    if (offers.isEmpty()) {
+        kDebug() << "fail";
+        return;
+    }
+
+    KUrl::List urls;
+    KService::Ptr service = offers.first();
+    KRun::run(*service, urls, 0);
+}
+
 void ClockApplet::wheelEvent(QGraphicsSceneWheelEvent *event)
 {
     if (d->selectedTimezones.count() < 1) {
@@ -555,40 +528,20 @@ void ClockApplet::initExtenderItem(Plasma::ExtenderItem *item)
     if (item->name() == "calendar") {
         item->setTitle(i18n("Calendar"));
         item->setIcon("view-pim-calendar");
-
-        Plasma::Calendar *calendar = new Plasma::Calendar;
-        calendar->setMinimumSize(QSize(230, 220));
-        calendar->setDataEngine(dataEngine("calendar"));
-        calendar->setRegion(d->displayHolidays ? d->holidaysRegion : QString());
-        connect(calendar, SIGNAL(dateChanged(const QDate &)), this, SLOT(dateChanged(const QDate &)));
-
-        Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
-        QDate date = data["Date"].toDate();
-        if (date.isValid()) {
-            calendar->setDate(date);
-        }
-
-        item->setWidget(calendar);
-
+        item->setWidget(d->calendarWidget);
     } else if (item->name() == "today") {
         item->setTitle(i18n("Today"));
         item->setIcon("view-pim-calendar");
         d->label = new Plasma::Label();
         item->setWidget(d->label);
-    } else if (item->name().startsWith("dateExtender-")) {
+    } else if (item->name().startsWith(QLatin1String("dateExtender-"))) {
         item->setIcon("view-pim-calendar");
         item->showCloseButton();
         QDate date = QDate::fromString(item->name().remove(0, 13), Qt::ISODate);
-        item->setTitle(KGlobal::locale()->formatDate(date));
-        Plasma::Calendar *calendar = d->calendarWidget();
         DateExtenderWidget *widget = 0;
 
-        if (calendar) {
-            QString holidayDesc = calendar->dateProperty(date);
-            widget = new DateExtenderWidget(holidayDesc);
-        } else {
-            widget = new DateExtenderWidget(date, dataEngine("calendar"), d->holidaysRegion);
-        }
+        item->setTitle(calendar()->formatDate(date));
+        widget = new DateExtenderWidget(d->calendarWidget->dateProperty(date));
 
         item->setWidget(widget);
     }
@@ -604,19 +557,20 @@ void ClockApplet::init()
 
     d->setPrettyTimezone();
 
-    QString tmpRegion = KGlobal::locale()->country();
-    if(!dataEngine("calendar")->query("holidaysRegions").value("holidaysRegions").toStringList().contains(tmpRegion)){
-        tmpRegion = QString::null;
-    }
-
-    d->displayHolidays = cg.readEntry("displayHolidays", false);
-    d->holidaysRegion = cg.readEntry("holidaysRegion", tmpRegion);
     d->announceInterval = cg.readEntry("announceInterval", 0);
 
     Plasma::ToolTipManager::self()->registerWidget(this);
 
+    d->calendarWidget = new Plasma::Calendar();
+    d->calendarWidget->setMinimumSize(QSize(230, 220));
+    d->calendarWidget->setDataEngine(dataEngine("calendar"));
+    connect(d->calendarWidget, SIGNAL(dateChanged(const QDate &)), this, SLOT(dateChanged(const QDate &)));
+    d->calendarWidget->applyConfiguration(cg);
+    Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
+    d->calendarWidget->setDate(data["Date"].toDate());
+    d->createCalendarExtender();
+
     extender();
-    d->createCalendar();
     QTimer::singleShot(0, this, SLOT(createToday()));
 }
 
@@ -626,14 +580,8 @@ void ClockApplet::popupEvent(bool show)
         return;
     }
 
-    Plasma::Calendar *calendar = d->calendarWidget();
-    if (calendar){
-        Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
-        QDate date = data["Date"].toDate();
-        if (date.isValid()) {
-            calendar->setDate(date);
-        }
-    }
+    Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
+    d->calendarWidget->setDate(data["Date"].toDate());
 
     d->destroyDateExtenders();
 }
@@ -700,18 +648,8 @@ void ClockApplet::dateChanged(const QDate &date)
 {
     d->destroyDateExtenders();
 
-    Plasma::Calendar *calendar = d->calendarWidget();
-
-    if (calendar) {
-        QString holidayDesc = calendar->dateProperty(date);
-        if (!holidayDesc.isEmpty()) {
-            d->createDateExtender(date);
-        }
-    } else {
-        QString tmpStr = "isHoliday:" + d->holidaysRegion + ":" + date.toString(Qt::ISODate);
-        if (dataEngine("calendar")->query(tmpStr).value(tmpStr).toBool()) {
-            d->createDateExtender(date);
-        }
+    if (!d->calendarWidget->dateProperty(date).isEmpty()) {
+        d->createDateExtender(date);
     }
 }
 
@@ -721,27 +659,52 @@ void ClockApplet::updateClipboardMenu()
     QList<QAction*> actions;
     Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
     QDateTime dateTime = QDateTime(data["Date"].toDate(), data["Time"].toTime());
-    KLocale *locale = KGlobal::locale();
 
-    d->clipboardMenu->addAction(locale->formatDate(dateTime.date(), KLocale::LongDate));
-    d->clipboardMenu->addAction(locale->formatDate(dateTime.date(), KLocale::ShortDate));
-    d->clipboardMenu->addAction(locale->formatDateTime(dateTime));
+    d->clipboardMenu->addAction(calendar()->formatDate(dateTime.date(), KLocale::LongDate));
+    d->clipboardMenu->addAction(calendar()->formatDate(dateTime.date(), KLocale::ShortDate));
+    // Display ISO Date format if not already displayed
+    if (KGlobal::locale()->dateFormatShort() != "%Y-%m-%d") {
+        d->clipboardMenu->addAction(calendar()->formatDate(dateTime.date(), "%Y-%m-%d"));
+    }
 
     QAction *sep0 = new QAction(this);
     sep0->setSeparator(true);
     d->clipboardMenu->addAction(sep0);
 
-    d->clipboardMenu->addAction(locale->formatTime(dateTime.time(), false));
-    d->clipboardMenu->addAction(locale->formatTime(dateTime.time(), true));
+    d->clipboardMenu->addAction(KGlobal::locale()->formatTime(dateTime.time(), false));
+    d->clipboardMenu->addAction(KGlobal::locale()->formatTime(dateTime.time(), true));
 
     QAction *sep1 = new QAction(this);
     sep1->setSeparator(true);
     d->clipboardMenu->addAction(sep1);
 
-    d->clipboardMenu->addAction(dateTime.date().toString());
-    d->clipboardMenu->addAction(dateTime.toString());
-    d->clipboardMenu->addAction(dateTime.toString("yyyy-MM-dd hh:mm:ss"));
-    d->clipboardMenu->addAction(dateTime.time().toString());
+    KLocale tempLocale(*KGlobal::locale());
+    tempLocale.setCalendar(calendar()->calendarType());
+    d->clipboardMenu->addAction(tempLocale.formatDateTime(dateTime, KLocale::LongDate));
+    d->clipboardMenu->addAction(tempLocale.formatDateTime(dateTime, KLocale::LongDate, true));
+    d->clipboardMenu->addAction(tempLocale.formatDateTime(dateTime, KLocale::ShortDate));
+    d->clipboardMenu->addAction(tempLocale.formatDateTime(dateTime, KLocale::ShortDate, true));
+    // Display ISO DateTime format if not already displayed
+    if (tempLocale.dateFormatShort() != "%Y-%m-%d") {
+        tempLocale.setDateFormatShort("%Y-%m-%d");
+        d->clipboardMenu->addAction(tempLocale.formatDateTime(dateTime, KLocale::ShortDate, true));
+    }
+
+    QAction *sep2 = new QAction(this);
+    sep2->setSeparator(true);
+    d->clipboardMenu->addAction(sep2);
+
+    QStringList calendars = KCalendarSystem::calendarSystems();
+    foreach ( const QString &cal, calendars ) {
+        if (cal != calendar()->calendarType()) {
+            KCalendarSystem *tempCal = KCalendarSystem::create(cal);
+            QString text = tempCal->formatDate(dateTime.date(), KLocale::LongDate) + " (" + KCalendarSystem::calendarLabel(cal) + ')';
+            d->clipboardMenu->addAction(text);
+            text = tempCal->formatDate(dateTime.date(), KLocale::ShortDate) + " (" + KCalendarSystem::calendarLabel(cal) + ')';
+            d->clipboardMenu->addAction(text);
+            delete tempCal;
+        }
+    }
 }
 
 void ClockApplet::copyToClipboard(QAction* action)
@@ -752,4 +715,9 @@ void ClockApplet::copyToClipboard(QAction* action)
     QApplication::clipboard()->setText(text);
 }
 
+const KCalendarSystem *ClockApplet::calendar() const
+{
+    return d->calendarWidget->calendar();
+}
+
 #include "clockapplet.moc"
