@@ -40,6 +40,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <KDebug>
 #include <KIconLoader>
 #include <KLocale>
+#include <NETWinInfo>
 
 #include "taskmanager.h"
 
@@ -163,7 +164,6 @@ void Task::refreshIcon()
             d->cachedChangesTimerId = startTimer(200 - d->lastUpdate.elapsed());
         }
 
-        d->lastUpdate.restart();
         return TaskUnchanged;
     }
 
